clc
clear all
close all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('*****  LOAD POLICY FUNCTIONS  *************')

stepx = 250;
stepy = 100;
stept = 30*12;

direc1 = '/Users/stefanocorradin/Stefano/Research/leverage/JMCB-HouseholdLeverage#09-451/CPlusMatlab_Code/BenchParam_College_Tab3_4/mup167vol10/ltv93/';

parameters = load(strcat(direc1,'Parameters.dat'));
ccxomGdat = load(strcat(direc1,'StorevaluesccxomG.dat'));
ccxomBdat = load(strcat(direc1,'StorevaluesccxomB.dat')); 
ccxrGdat = load(strcat(direc1,'StorevaluesccxrG.dat'));
ccxrBdat = load(strcat(direc1,'StorevaluesccxrB.dat'));
ominxdGdat = load(strcat(direc1,'StorevaluesominxdG.dat'));
ominxdBdat = load(strcat(direc1,'StorevaluesominxdB.dat'));

ccxomG = zeros(stepx,stepy,stept);
ccxomB = zeros(stepx,stepy,stept);
ccxrG = zeros(stepx,stepy,stept);
ccxrB = zeros(stepx,stepy,stept);
ominxdG = zeros(stepx,stepy,stept);
ominxdB = zeros(stepx,stepy,stept);

for time = stept-1:-1:1

    for j = 1:stepy
        
        for i = 1:stepx
            
            ind = i + (j-1)*stepx; 
            pos = ind + stepx*stepy*(time - 1);
            ccxomG(i,j,time) = ccxomGdat(pos);
            ccxomB(i,j,time) = ccxomBdat(pos);
            ccxrG(i,j,time) = ccxrGdat(pos);
            ccxrB(i,j,time) = ccxrBdat(pos);
            ominxdG(i,j,time) = ominxdGdat(pos);
            ominxdB(i,j,time) = ominxdBdat(pos);
            
        end    
        
    end    
    
end    
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main parameters
age = parameters(1);		
T = parameters(2);			
gamma =	parameters(3);		
beta = parameters(4);	
rho = parameters(5);
r = parameters(6);	
mup = parameters(7);			
sigmap = parameters(8);		
H = parameters(9);			
LG0 = parameters(10);		
muL0 =	parameters(11);
muL1 = parameters(12);
muL2 =	parameters(13);
muL3 = parameters(14);
LBfrac = parameters(15);		       
thetaG = parameters(16);	
thetaB = parameters(17); 
kappa = parameters(18);		
dead =	parameters(19);                    
epsilon =	parameters(20);
taxrate	= parameters(21);
taxhouse = parameters(22);
delta = parameters(23);
spread = parameters(24);
lambda = parameters(25);

% Other parameters
alpha = r - mup + lambda + taxhouse + delta;
Mo = (1 - kappa)*H;
R = r + spread;	
Cou = R*Mo/(1 - exp(- R*T));
Wo = LG0;
Wom = LG0 - kappa*H;

Months = 12;
dt = 1/Months;

for time = stept-1:-1:1 
	 
	 LG(time) = (1 - taxrate)*LG0*exp( muL0 + muL1*(time*dt + age) + muL2*( power(time*dt+age,2)/10 ) + muL3*( power(time*dt+age,3)/100 ) );
	 LB(time) = LG(time)*LBfrac;
     
     phi(time) = (R*exp(- R*(T - time*dt)))/(1 - exp(- R*(T - time*dt)));
     fac(time) = (1 - exp(- R*(T - time*dt)))/(1 - exp(- R*T ));
     Mt(time) = fac(time)*Mo;

end	
        
wmax = 250;
wmin = 0.01;
hw = (wmax - wmin)/stepx; 
x = wmin:hw:wmax-hw;
pmax = 2;
pmin = 0.01;
hp = (pmax - pmin)/stepy;
y = pmin:hp:pmax-hp;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('******** SIMULATION *********************')

NSim = 10000;
NSteps = T*Months;
PPaths = zeros(NSim,NSteps);
PPaths(:,1) = ones(NSim,1);
LPaths = zeros(NSim,NSteps);

% House prices
muPdt = (mup - 0.5*sigmap^2)*dt;
siPdt = sigmap*sqrt(dt);
dz1 = randn(NSim,NSteps);

for j = 1:NSteps-1

    PPaths(:,j+1) = PPaths(:,j).*exp( muPdt + siPdt.*dz1(:,j) );
    
end

% Labor income
regimes = 2;
Pmarkov = [(1 - thetaG)  thetaG; thetaB  (1 - thetaB)];
Qmarkov = Pmarkov - eye(regimes,regimes) - mpower( Pmarkov - eye(regimes,regimes),2)./2  + ...
                mpower( Pmarkov - eye(regimes,regimes),3)./3 - mpower( Pmarkov - eye(regimes,regimes),4)./4;
Pmarkovdt = expm(Qmarkov.*Months);
xmarkov = zeros(NSim,NSteps);
regimeslab = (1:1:regimes)';
%Pmarkovdt = Pmarkov;

for j=1:NSim
   
    xmarkov(j,:) = simulate_markov(regimeslab,Pmarkovdt,[1 0],T/dt);
        
end

xom = zeros(NSim,NSteps);
cons = zeros(NSim,NSteps);
b10pi = zeros(NSim,NSteps);
b10ni = zeros(NSim,NSteps);
xom(:,1) = Wom.*ones(NSim,1);
V = zeros(NSim,NSteps);
default = zeros(NSim,1);

for i=1:NSim

    for j = 1:1:NSteps-1
        
        [ix,jx] = find(xom(i,j) < x);
        [iy,jy] = find(PPaths(i,j) < y);
        
        if isempty(jy) == 1 && PPaths(i,j) > pmax - hp
           
           jy(1) = stepy; 
            
        end    
        
        posx = jx(1);
        posy = jy(1);
        pos(i,j,1:2) = [posx posy];
        
        if ominxdG(posx,posy,j) == 0 && default(i) == 0
           
           default(i) = 1;
           deltax = epsilon*H*PPaths(i,j);
           
        elseif ominxdG(posx,posy) == 0 && default(i) == 1
                
           deltax = 0;   
            
        end    
        
        if xmarkov(i,j) == 1 && default(i) == 0
            
           L = LG(j);
           cons(i,j) = ccxomG(posx,posy,j);          
           
        elseif xmarkov(i,j) == 2 && default(i) == 0
            
           L = LB(j);
           cons(i,j) = ccxomB(posx,posy,j);
           
        elseif xmarkov(i,j) == 1 && default(i) == 1
            
           L = LG(j);
           cons(i,j) = ccxrG(posx,posy,j);          
           
        elseif xmarkov(i,j) == 2 && default(i) == 1
            
           L = LB(j);
           cons(i,j) = ccxrB(posx,posy,j);   
           
        end      
        
        if default(i) == 0
            
            b10pi(i,j) =  r*xom(i,j) + taxhouse*taxrate*H*PPaths(i,j) +  taxrate*Cou + L;
            b10ni(i,j) = Cou + (taxhouse + delta)*H*PPaths(i,j) + cons(i,j) + taxrate*phi(j)*Mt(j);
            xom(i,j+1) = xom(i,j) + ( b10pi(i,j) - b10ni(i,j) ).*dt;
            
        elseif default(i) == 1
            
            b10pi(i,j) =  r*xom(i,j) + L;
            b10ni(i,j) = alpha*H*PPaths(i,j) + cons(i,j);
            xom(i,j+1) = xom(i,j) + ( b10pi(i,j) - b10ni(i,j) ).*dt - deltax;
            
        end    
        
    end
    
end

for i = 1:NSim

    if default(i) == 0
    
        cons(i,end) = PPaths(i,end)*H + xom(i,end);
        
    elseif default(i) == 1 
       
        cons(i,end) = xom(i,end);
        
    end    

end

V(:,end) = power(cons(:,end), 1 - gamma)./(1 - gamma);

for j = NSteps-1:-1:1
   
    uc = power( power(cons(:,j),beta).*power(H,1 - beta), 1 - gamma)./(1 - gamma);
    V(:,j) = uc*dt + exp( - rho*dt).*V(:,j+1);
    
end    

EV = mean(V(:,1));
Pr1_V = prctile(V(:,1),1);
Pr99_V = prctile(V(:,1),99);

disp('**********************   RESULTS   **********************************')
fprintf('Down payment = %f\n',kappa*100 );
fprintf('Average - V = %f\n',EV);
fprintf('Perc. 1 - V = %f\n',Pr1_V);
fprintf('Perc. 99 - V = %f\n',Pr99_V);

sV = sort(V(:,1));
int = 100/NSim;
vecint = 1+int:int:100;

figure(1)
plot(vecint,sV(sV > Pr1_V),'--')

matres = zeros(1,4);
matres(1,1) = kappa*100;
matres(1,2) = Pr1_V;
matres(1,3) = EV;
matres(1,4) = Pr99_V;

save(strcat(direc1,'output'),'matres')