
/* 

Corradin Stefano
Household Leverage 

cd Stefano/Research/leverage/JMCB-HouseholdLeverage#09-451/CPlusMatlab_Code/BenchParam_College_Tab3_4/mup167vol10/ltv93/

*/

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<string.h>

#define abs_d(a)	((a > 0)?a:-a)
#define min_d(a,b)	((a < b)?a:b)
#define max_d(a,b)	((a > b)?a:b)
#define myvar double
#define MYSTEPX 250
#define MYSTEPY 100
				
				
void gridgen(	int stepx, int stepy, myvar hw, myvar hp, 
				myvar wmin, myvar wmax,
				myvar pmin, myvar pmax,
				myvar *w, myvar *p, myvar H,
				myvar *xom, myvar *xr, myvar *y, 
				int *ominx, int *ominxdG, int *ominxdB, int *ominy, int *rinx, int *riny 
				)
{
	int i,j,ind;
	
	w[1] = wmin;
	for (i = 1; i <= stepx - 1; i++)
	{
		w[i+1] = w[i] + hw;
	}
			    
	p[1] = pmin;
	for (i = 1; i <= stepy - 1; i++)
	{
		p[i+1] = p[i] + hp;
	}


	for(i = 1; i <= stepx; i++)
	{
		for(j = 1; j <= stepy; j++)
        {
			ind = i + (j-1)*stepx;
			xom[ind] = w[i];
			xr[ind] = w[i];
			y[ind] = p[j];
			ominx[ind] = i;
			ominxdG[ind] = i;
			ominxdB[ind] = i;
			ominy[ind] = j;
			rinx[ind] = i;
			riny[ind] = j;
        }
	}    
}


void rnfpmarkov( myvar r, myvar delta, myvar taxhouse, myvar lambda,  
				 myvar mup, myvar sigmap, 
				 int stepx, int stepy, myvar invtau, myvar invhw, myvar invhp,
				 myvar *xr, myvar *y, myvar H,
				 myvar *Qfr, myvar *nfpxphyr, myvar *nfpxnhyr, myvar *npxypgr, myvar *npxyngr, 
				 myvar *npxphypgr, myvar *npxnhyngr, myvar *npxphyngr, myvar *npxnhypgr)
{
		
	int i;
	myvar b20i;
	myvar b02i;
	myvar b11i;
	myvar fb10pi;		
	myvar fb10ni;		
	myvar b01pi;
	myvar b01ni;
		
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		
		/* First Derivative */
		fb10pi = invhw*( r*xr[i] +  mup*y[i]*H);
		fb10ni = invhw*( (r + lambda + delta + taxhouse)*H*y[i] );
		b01pi = invhp*( mup*y[i] ); 
		b01ni = invhp*( 0 );
          
		/* Second derivative */
		b20i = 0;
		b02i = invhp*invhp*( sigmap*sigmap*y[i]*y[i] );
		b11i = 0;		
                		
		if(b11i > 0)
		{
			/* p[x+h,y+g] */
			npxphypgr[i] = 0.5*b11i;
			/* p[x-h,y-g] */
			npxnhyngr[i] = npxphypgr[i];
			/* p[x+h,y-g] */
			npxphyngr[i] = 0;
			/* p[x-h,y+g] */
			npxnhypgr[i] = npxphyngr[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypgr[i] = 0;
			/* p[x-h,y-g] */
			npxnhyngr[i] = npxphypgr[i];
			/* p[x+h,y-g] */
			npxphyngr[i] = - 0.5*b11i;
			/* p[x-h,y+g] */
			npxnhypgr[i] = npxphyngr[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		nfpxphyr[i] = fb10pi + 0.5*b20i - 0.5*abs_d(b11i);        
		/* p[x-h,y] */
		nfpxnhyr[i] = fb10ni + 0.5*b20i - 0.5*abs_d(b11i);
		/* p[x,y+g] */
		npxypgr[i] = b01pi + 0.5*b02i - 0.5*abs_d(b11i);
		/* p[x,y-g] */
		npxyngr[i] = b01ni + 0.5*b02i - 0.5*abs_d(b11i);
		/* Qfixed */ 
		Qfr[i] = invtau + fb10ni + b01pi + b01ni + fb10pi + b20i + b02i -  abs_d(b11i);
		 
	}
}								

void omnfpmarkov( myvar r, myvar mup, myvar sigmap, 
				 int stepx, int stepy, myvar invtau, myvar invhw, myvar invhp,
				 myvar *xom, myvar *y, myvar H, myvar Cou,  myvar delta, myvar taxhouse, myvar taxrate, 
				 myvar *Qfom, myvar *nfpxphyom, myvar *nfpxnhyom, myvar *npxypgom, myvar *npxyngom, 
				 myvar *npxphypgom, myvar *npxnhyngom, myvar *npxphyngom, myvar *npxnhypgom)
{
		
	int i;
	myvar b20i;
	myvar b02i;
	myvar b11i;
	myvar fb10pi;		
	myvar fb10ni;		
	myvar b01pi;
	myvar b01ni;
		
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		
		/* First Derivative */
		fb10pi = invhw*( r*xom[i] + taxhouse*taxrate*H*y[i] );
		fb10ni = invhw*( Cou + (taxhouse + delta)*H*y[i]  );
		b01pi = invhp*( mup*y[i] ); 
		b01ni = invhp*( 0 );
          
		/* Second derivative */
		b20i = 0;
        b02i = invhp*invhp*( sigmap*sigmap*y[i]*y[i] );		
        b11i = 0;
						        		
		if(b11i > 0)
		{
			/* p[x+h,y+g] */
			npxphypgom[i] = 0.5*b11i;
			/* p[x-h,y-g] */
			npxnhyngom[i] = npxphypgom[i];
			/* p[x+h,y-g] */
			npxphyngom[i] = 0;
			/* p[x-h,y+g] */
			npxnhypgom[i] = npxphyngom[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypgom[i] = 0;
			/* p[x-h,y-g] */
			npxnhyngom[i] = npxphypgom[i];
			/* p[x+h,y-g] */
			npxphyngom[i] = - 0.5*b11i;
			/* p[x-h,y+g] */
			npxnhypgom[i] = npxphyngom[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		nfpxphyom[i] = fb10pi + 0.5*b20i - 0.5*abs_d(b11i);        
		/* p[x-h,y] */
		nfpxnhyom[i] = fb10ni + 0.5*b20i - 0.5*abs_d(b11i);
		/* p[x,y+g] */
		npxypgom[i] = b01pi + 0.5*b02i - 0.5*abs_d(b11i);
		/* p[x,y-g] */
		npxyngom[i] = b01ni + 0.5*b02i - 0.5*abs_d(b11i);
		/* Qfixed */ 
		Qfom[i] = invtau + fb10pi + fb10ni + b01pi + b01ni + b20i + b02i - abs_d(b11i);
		 
	}
}								

void mornfpmarkov( myvar r, myvar mup, myvar sigmap, 
				 int stepx, int stepy, myvar invtau, myvar invhw, myvar invhp,
				 myvar *xom, myvar *y, myvar H, myvar Cou, myvar delta, myvar taxhouse, myvar taxrate,
				 myvar *Qfm, myvar *nfpxphym, myvar *nfpxnhym, myvar *npxypgm, myvar *npxyngm, 
				 myvar *npxphypgm, myvar *npxnhyngm, myvar *npxphyngm, myvar *npxnhypgm)
{
		
	int i;
	myvar b20i;
	myvar b02i;
	myvar b11i;
	myvar fb10pi;		
	myvar fb10ni;		
	myvar b01pi;
	myvar b01ni;
		
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		
		/* First Derivative */
		fb10pi = invhw*( r*xom[i] + taxhouse*taxrate*H*y[i] );
		fb10ni = invhw*( Cou + (taxhouse + delta)*H*y[i]  );
		b01pi = invhp*( mup*y[i] ); 
		b01ni = invhp*( 0 );
          
		/* Second derivative */
		b20i = 0;
		b02i = invhp*invhp*(  sigmap*sigmap*y[i]*y[i]  );		
		b11i = 0;
						        		
		if(b11i > 0)
		{
			/* p[x+h,y+g] */
			npxphypgm[i] = 0.5*b11i;
			/* p[x-h,y-g] */
			npxnhyngm[i] = npxphypgm[i];
			/* p[x+h,y-g] */
			npxphyngm[i] = 0;
			/* p[x-h,y+g] */
			npxnhypgm[i] = npxphyngm[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypgm[i] = 0;
			/* p[x-h,y-g] */
			npxnhyngm[i] = npxphypgm[i];
			/* p[x+h,y-g] */
			npxphyngm[i] = - 0.5*b11i;
			/* p[x-h,y+g] */
			npxnhypgm[i] = npxphyngm[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		nfpxphym[i] = fb10pi + 0.5*b20i - 0.5*abs_d(b11i);        
		/* p[x-h,y] */
		nfpxnhym[i] = fb10ni + 0.5*b20i - 0.5*abs_d(b11i);
		/* p[x,y+g] */
		npxypgm[i] = b01pi + 0.5*b02i - 0.5*abs_d(b11i);
		/* p[x,y-g] */
		npxyngm[i] = b01ni + 0.5*b02i - 0.5*abs_d(b11i);
		/* Qfixed */ 
		Qfm[i] = invtau + fb10pi + fb10ni + b01pi + b01ni + b20i + b02i - abs_d(b11i);
		 
	}
}				

void pdnfpmarkov( myvar r, myvar mup, myvar sigmap, 
				 int stepx, int stepy, myvar invtau, myvar invhw, myvar invhp,
				 myvar *xom, myvar *y, myvar H, myvar Cou, myvar delta, myvar taxhouse, myvar taxrate,
				 myvar *Qfm, myvar *nfpxphym, myvar *nfpxnhym, myvar *npxypgm, myvar *npxyngm, 
				 myvar *npxphypgm, myvar *npxnhyngm, myvar *npxphyngm, myvar *npxnhypgm)
{
		
	int i;
	myvar b20i;
	myvar b02i;
	myvar b11i;
	myvar fb10pi;		
	myvar fb10ni;		
	myvar b01pi;
	myvar b01ni;
		
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		
		/* First Derivative */
		fb10pi = invhw*( r*xom[i] + taxhouse*taxrate*H*y[i] );
		fb10ni = invhw*( Cou + (taxhouse + delta)*H*y[i]  );
		b01pi = invhp*( mup*y[i] ); 
		b01ni = invhp*( 0 );
          
		/* Second derivative */
		b20i = 0;
               b02i = invhp*invhp*(  sigmap*sigmap*y[i]*y[i]  );		
               b11i = 0;
						        		
		if(b11i > 0)
		{
			/* p[x+h,y+g] */
			npxphypgm[i] = 0.5*b11i;
			/* p[x-h,y-g] */
			npxnhyngm[i] = npxphypgm[i];
			/* p[x+h,y-g] */
			npxphyngm[i] = 0;
			/* p[x-h,y+g] */
			npxnhypgm[i] = npxphyngm[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypgm[i] = 0;
			/* p[x-h,y-g] */
			npxnhyngm[i] = npxphypgm[i];
			/* p[x+h,y-g] */
			npxphyngm[i] = - 0.5*b11i;
			/* p[x-h,y+g] */
			npxnhypgm[i] = npxphyngm[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		nfpxphym[i] = fb10pi + 0.5*b20i - 0.5*abs_d(b11i);        
		/* p[x-h,y] */
		nfpxnhym[i] = fb10ni + 0.5*b20i - 0.5*abs_d(b11i);
		/* p[x,y+g] */
		npxypgm[i] = b01pi + 0.5*b02i - 0.5*abs_d(b11i);
		/* p[x,y-g] */
		npxyngm[i] = b01ni + 0.5*b02i - 0.5*abs_d(b11i);
		/* Qfixed */ 
		Qfm[i] = invtau + fb10pi + fb10ni + b01pi + b01ni + b20i + b02i - abs_d(b11i);
		 
	}
}					

void valuerent(	myvar type, myvar gamma, myvar beta, myvar L, myvar H, myvar r, 
			            myvar invtau, int stepx, int stepy, myvar invhw,  myvar rho,  
			          int *inx, int *iny,
			         myvar **V, myvar **Vold, myvar *Vtime, myvar **Voldss, myvar theta,
					 myvar *uc, myvar *ia, myvar *ccx,
			         myvar *Qf, myvar *dtxi, myvar *disci,
				myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
			     myvar *npxypg, myvar *npxyng, 
			   myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
			       myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
				myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
			    myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prtau;
	myvar prxy;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar felicity;
	myvar depuc;
	myvar b10pQi;
	myvar b10pi;
	myvar b10nQi;
	myvar b10ni;						
	myvar Qsi;
	myvar invQsi;  
	
	// Step iteration
		for (i = 1; i <= stepx*stepy; i++)
		{		
				depuc = pow(ccx[i],beta)*pow(H,(1-beta)); 
				uc[i] = pow( depuc,(1 - gamma) )/(1 - gamma);						
				b10pQi = invhw*L;
				b10pi = invhw*L;
				b10nQi = invhw*ccx[i];
				b10ni =  invhw*ccx[i];						
				Qsi = b10pQi + b10nQi + Qf[i];
				npxnhy[i] = b10ni + nfpxnhy[i];
				npxphy[i] = b10pi + nfpxphy[i];
				invQsi = 1/Qsi;
			        dtxi[i] = invQsi;
				disci[i] = exp(-rho*dtxi[i]);
						
				if (inx[i] == 1 && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxy[i] = 1 - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}				
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
		  	        else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy )
				{
				        ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 -  pxypg[i] - pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else if (inx[i] == 1 && iny[i] == stepy)
				{
				        ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 - pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == stepy)																														
				{	
					ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];					
					pxnhy[i] = invQsi*npxnhy[i];
					pxy[i] = 1 - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
					
				}
				else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
								pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == 1) 
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == stepy)  
				{
					ptau[i] = invQsi*invtau;
					pxnhy[i] = invQsi*npxnhy[i];
					pxy[i] = 1 - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxphyng[i] = invQsi*npxphyng[i];
					pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}

							
				prxy = disci[i]*pxy[i];
				prtau = disci[i]*ptau[i];
				prxphy = disci[i]*pxphy[i];
				prxnhy = disci[i]*pxnhy[i];
				prxypg = disci[i]*pxypg[i];
				prxyng = disci[i]*pxyng[i];
				prxphypg = disci[i]*pxphypg[i];
				prxnhyng = disci[i]*pxnhyng[i];
				prxnhypg = disci[i]*pxnhypg[i];
				prxphyng = disci[i]*pxphyng[i];      
				felicity = dtxi[i]*(uc[i] + theta*( Voldss[inx[i]+1][iny[i]+1] - Vold[inx[i]+1][iny[i]+1] )) ;
					
                V[inx[i]+1][iny[i]+1] = felicity + prtau*Vtime[i] + prxy*Vold[inx[i]+1][iny[i]+1] +  
											prxphy*Vold[inx[i]+2][iny[i]+1] + 
											prxnhy*Vold[inx[i]][iny[i]+1] +  
											prxypg*Vold[inx[i]+1][iny[i]+2] + 
											prxyng*Vold[inx[i]+1][iny[i]] +
											prxphypg*Vold[inx[i]+2][iny[i]+2] +
											prxnhyng*Vold[inx[i]][iny[i]] +
											prxnhypg*Vold[inx[i]][iny[i]+2] +
											prxphyng*Vold[inx[i]+2][iny[i]];
									
	
	}
}

void consrent(	myvar mup, myvar sigmap, myvar gamma, 
						myvar alpha, myvar beta, myvar H, myvar r, myvar rho,
						myvar tau, myvar L, myvar invtau,
						myvar *ccons, myvar *cfac,
						myvar **ccx, myvar **ccxold, myvar *ccxtime, myvar **ccxss, myvar theta,
						int stepx, int stepy, myvar invhw, myvar invhp, 
						int *inx, int *iny, myvar *xr, myvar *y,
						myvar *uc, myvar *ia, 
						myvar *Qf, myvar *dtxi, myvar *disci,
						myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
						myvar *npxypg, myvar *npxyng, 
						myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
						myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
						myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
						myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prtau;
        myvar prxy;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar felicity;
	myvar depuc;
	myvar Qsi;
	myvar invQsi;  
	myvar b001;
	myvar b100p;
	myvar b100n;
	myvar b010p;
	myvar b010n;
	myvar b200;
	myvar b020;
	myvar b110;
			
	for (i = stepx*stepy; i > 0; i--)
	{ 
		b001 = 1 - beta + beta*gamma;
		
		b100p = invhw*( beta*ccons[i] + L + beta*gamma*L + r*xr[i] + 
							beta*gamma*r*xr[i] + beta*H*y[i]*alpha );
		
		b100n = invhw*( ccons[i] + beta*ccons[i]*gamma + beta*L + beta*r*xr[i] + H*y[i]*alpha + 
							beta*gamma*H*y[i]*alpha );
		
		b010p = invhp*( mup*y[i] + beta*gamma*mup*y[i] ); 
		
		b010n = invhp*( beta*mup*y[i] );
          
		/* Second derivative */
		b200 = 0;
		
		b020 = invhp*invhp*(	0.5*y[i]*y[i]*sigmap*sigmap - 
								0.5*beta*y[i]*y[i]*sigmap*sigmap +  
								0.5*beta*gamma*y[i]*y[i]*sigmap*sigmap );
		
		b110 = 0;		
                		
		if(b110 > 0)
		{
			/* p[x+h,y+g] */
			npxphypg[i] = 0.5*b110;
			/* p[x-h,y-g] */
			npxnhyng[i] = npxphypg[i];
			/* p[x+h,y-g] */
			npxphyng[i] = 0;
			/* p[x-h,y+g] */
			npxnhypg[i] = npxphyng[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypg[i] = 0;
			/* p[x-h,y-g] */
			npxnhyng[i] = npxphypg[i];
			/* p[x+h,y-g] */
			npxphyng[i] = - 0.5*b110;
			/* p[x-h,y+g] */
			npxnhypg[i] = npxphyng[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		npxphy[i] = b100p + b200 - 0.5*abs(b110);        
        /* p[x-h,y] */
		npxnhy[i] = b100n + b200 - 0.5*abs(b110);
		/* p[x,y+g] */
		npxypg[i] = b010p + b020 - 0.5*abs(b110);
		/* p[x,y-g] */
		npxyng[i] = b010n + b020 - 0.5*abs(b110);
		/* Qfixed */ 
		Qf[i] = b001*invtau + b100p + b100n + b010p + b010n + 2*b200 + 2*b020 - abs(b110);
		 
		 
		uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
                   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);						
		
		invQsi = 1/Qf[i];
		dtxi[i] = invQsi; 
		disci[i] = 1;
						
		if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
		{
			ptau[i] = invQsi*invtau*b001;
			pxypg[i] = invQsi*npxypg[i];
			pxphy[i] = invQsi*npxphy[i];
			pxnhy[i] = invQsi*npxnhy[i];
			pxphypg[i] = invQsi*npxphypg[i];
			pxnhypg[i] = invQsi*npxnhypg[i];
			pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
						pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}
		else if (inx[i] == 1 && iny[i] == 1)																														
		{	
			ptau[i] = invQsi*invtau*b001;
			pxy[i] = 1 - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = 0;
		}	
		else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy)
		{
			ptau[i] = invQsi*invtau*b001;
			pxy[i] = 1 - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = 0;
			
		
		}
		else if (inx[i] == 1 && iny[i] == stepy)
		{
			ptau[i] = invQsi*invtau;
			pxy[i] = 1 - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = 0;
			
		}
		
		else if (inx[i] > 1 && inx[i] < stepx && iny[i] == stepy)																														
		{	
			
			ptau[i] = invQsi*invtau*b001;
			pxphy[i] = invQsi*npxphy[i];
			pxnhy[i] = invQsi*npxnhy[i];
			pxy[i] = 1 - pxphy[i] - pxnhy[i] - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
			
		}
		else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
		{
			ptau[i] = invQsi*invtau*b001;
			pxypg[i] = invQsi*npxypg[i];
			pxnhy[i] = invQsi*npxnhy[i];
			pxyng[i] = invQsi*npxyng[i];
			pxnhyng[i] = invQsi*npxnhyng[i];
			pxnhypg[i] = invQsi*npxnhypg[i];
			pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
							pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}
		else if (inx[i] == stepx && iny[i] == 1) 
		{
			ptau[i] = invQsi*invtau*b001;
			pxypg[i] = invQsi*npxypg[i];
			pxnhy[i] = invQsi*npxnhy[i];
			pxnhypg[i] = invQsi*npxnhypg[i];
			pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}
		else if (inx[i] == 1 && iny[i] == stepy)  
		{
			ptau[i] = invQsi*invtau*b001;
			pxy[i] = 1 - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}	
		else if (inx[i] == stepx && iny[i] == stepy)  
		{
			
			ptau[i] = invQsi*invtau*b001;
			pxnhy[i] = invQsi*npxnhy[i];
			pxy[i] = 1  - pxnhy[i]  - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}
		else
		{
			ptau[i] = invQsi*invtau*b001;
			pxypg[i] = invQsi*npxypg[i];
			pxphy[i] = invQsi*npxphy[i];
			pxnhy[i] = invQsi*npxnhy[i];
			pxyng[i] = invQsi*npxyng[i];
			pxphypg[i] = invQsi*npxphypg[i];
			pxnhyng[i] = invQsi*npxnhyng[i];
			pxnhypg[i] = invQsi*npxnhypg[i];
			pxphyng[i] = invQsi*npxphyng[i];
			pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
			invpr[i] = 1/(1 - disci[i]*pxy[i]);
			uc[i] = (-(cfac[i]*(2 + beta*(3 + beta*(-1 + gamma))*(-1 + gamma))*
					   y[i]*y[i]*sigmap*sigmap) + 2*ccons[i]*ccons[i]*(-r + rho))/(2*ccons[i]);
		}
						
		prtau = disci[i]*ptau[i];
        prxy = disci[i]*pxy[i];
		prxphy = disci[i]*pxphy[i];
		prxnhy = disci[i]*pxnhy[i];
		prxypg = disci[i]*pxypg[i];
		prxyng = disci[i]*pxyng[i];
		prxphypg = disci[i]*pxphypg[i];
		prxnhyng = disci[i]*pxnhyng[i];
		prxnhypg = disci[i]*pxnhypg[i];
		prxphyng = disci[i]*pxphyng[i];      
		
		
		felicity = dtxi[i]*invpr[i]*( uc[i] + theta*(ccxold[inx[i]+1][iny[i]+1]
					- pow(ccxss[inx[i]+1][iny[i]+1],-1+beta-beta*gamma)*pow(ccxold[inx[i]+1][iny[i]+1],2-beta+beta*gamma) ) ) ;
					
															
		ccx[inx[i]+1][iny[i]+1] = min_d( felicity + prtau*ccxtime[i] +  prxy*ccxold[inx[i]+1][iny[i]+1] + 
											prxphy*ccxold[inx[i]+2][iny[i]+1] + 
											prxnhy*ccxold[inx[i]][iny[i]+1] +  
											prxypg*ccxold[inx[i]+1][iny[i]+2] + 
											prxyng*ccxold[inx[i]+1][iny[i]] +
											prxphypg*ccxold[inx[i]+2][iny[i]+2] +
											prxnhyng*ccxold[inx[i]][iny[i]] +
											prxnhypg*ccxold[inx[i]][iny[i]+2] +
											prxphyng*ccxold[inx[i]+2][iny[i]], ia[i] );
	
	}
}

void valueown(	myvar type, myvar gamma, myvar beta, myvar L, myvar H, myvar r, 
			         myvar div, myvar taxrate,
		                 myvar wmax, myvar invtau,
		                 int stepx, int stepy, myvar invhw, myvar phi, myvar rho, myvar Mt, myvar *y,  myvar *xom,
			          int *inx, int *inxd, int *iny,
			         myvar **V, myvar **Vold, myvar *Vtime, myvar **Voldss, myvar theta,
					 myvar *uc, myvar *ia, myvar **ccx,
			         myvar *Qf, myvar *dtxi, myvar *disci,
				myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
			     myvar *npxypg, myvar *npxyng, 
			   myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
			   myvar *felicity, myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
				myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
			    myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prxy;
	myvar prtau;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar depuc;
	myvar b10pQi;
	myvar b10pi;
	myvar b10nQi;
	myvar b10ni;						
	myvar Qsi;
	myvar invQsi;  
	
	// Step iteration
		for (i = 1; i <= stepx*stepy; i++)
		{		
		
			if (inxd[i] == 0) 
        	{
				continue;
    		}
    		else  	
			{	
				depuc = pow(ccx[inx[i]+1][iny[i]+1],beta)*pow(H,(1-beta)); 
				uc[i] = pow( depuc,(1 - gamma) )/(1 - gamma);						
				b10pQi = invhw*( L + taxrate*div);
				b10pi =  invhw*( L + taxrate*div);
				b10nQi = invhw*( ccx[inx[i]+1][iny[i]+1] + taxrate*phi*Mt );
				b10ni =  invhw*( ccx[inx[i]+1][iny[i]+1] + taxrate*phi*Mt );						
				Qsi = b10pQi + b10nQi + Qf[i];
				npxnhy[i] = b10ni + nfpxnhy[i];
				npxphy[i] = b10pi + nfpxphy[i];
				invQsi = 1/Qsi;
			        dtxi[i] = invQsi;
				disci[i] = exp(-rho*dtxi[i]);
						
				if (inx[i] == 1 && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxy[i] = 1 - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}				
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
			  					pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
  	                        else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy )
				{
				        ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 -  pxypg[i] - pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else if (inx[i] == 1 && iny[i] == stepy)
				{
				        ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 -  pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == stepy)																														
				{	
					ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxyng[i] = invQsi*npxyng[i];					
					pxnhy[i] = invQsi*npxnhy[i];
					pxy[i] = 1 - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
					
				}
				else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
			 		pxyng[i] = invQsi*npxyng[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
								pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == 1) 
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == stepy)  
				{
					ptau[i] = invQsi*invtau;
					pxnhy[i] = invQsi*npxnhy[i];
					pxy[i] = 1 - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else 
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxphyng[i] = invQsi*npxphyng[i];
					pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
								
			        prxy = disci[i]*pxy[i];
				prtau = disci[i]*ptau[i];
				prxphy = disci[i]*pxphy[i];
				prxnhy = disci[i]*pxnhy[i];
				prxypg = disci[i]*pxypg[i];
				prxyng = disci[i]*pxyng[i];
				prxphypg = disci[i]*pxphypg[i];
				prxnhyng = disci[i]*pxnhyng[i];
				prxnhypg = disci[i]*pxnhypg[i];
				prxphyng = disci[i]*pxphyng[i];      
				felicity[i] = dtxi[i]*(uc[i] + theta*( Voldss[inx[i]+1][iny[i]+1] - Vold[inx[i]+1][iny[i]+1] )) ;
					
			
                                    V[inx[i]+1][iny[i]+1] = felicity[i] + prtau*Vtime[i] + prxy*Vold[inx[i]+1][iny[i]+1] +  
											prxphy*Vold[inx[i]+2][iny[i]+1] + 
											prxnhy*Vold[inx[i]][iny[i]+1] +  
											prxypg*Vold[inx[i]+1][iny[i]+2] + 
											prxyng*Vold[inx[i]+1][iny[i]] +
											prxphypg*Vold[inx[i]+2][iny[i]+2] +
											prxnhyng*Vold[inx[i]][iny[i]] +
											prxnhypg*Vold[inx[i]][iny[i]+2] +
											prxphyng*Vold[inx[i]+2][iny[i]];
			  
			  
				
			}	
	}
}


void consown(	myvar mup, myvar sigmap, myvar gamma, 
				myvar alpha, myvar beta, myvar H, myvar r, myvar rho,
				myvar tau, myvar L, myvar invtau,
		                myvar F, myvar phi, myvar Cou, myvar taxrate, myvar taxhouse, myvar delta,
				myvar *ccons, myvar *rcconsx, myvar *rcconsy, 
				myvar **ccx, myvar **ccxold, myvar *ccxtime, myvar** ccxss, myvar theta,
				int stepx, int stepy, myvar invhw, myvar invhp, 
				int *inx, int *inxd, int *iny, myvar *xom, myvar *y,
                                myvar *uc, myvar *ia, 
				myvar *Qf, myvar *dtxi, myvar *disci,
				myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
				myvar *npxypg, myvar *npxyng, 
				myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
				myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
				myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
				myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prtau;
        myvar prxy;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar felicity;
	myvar depuc;
	myvar Qsi;
	myvar invQsi;  
	myvar b001;
	myvar b100p;
	myvar b100n;
	myvar b010p;
	myvar b010n;
	myvar b200;
	myvar b020;
	myvar b110;
			
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		if (inxd[i] == 0) 
		{
			continue;
		}
		else  	
		{
		
		b001 = (1 - beta + beta*gamma);
		
		b100p = invhw*( Cou*taxrate + L + r*xom[i] + taxrate*taxhouse*H*y[i] 
				+ beta*(ccons[i] + Cou + Cou*taxrate*gamma + gamma*(L + r*xom[i] +  taxrate*taxhouse*H*y[i] ) 
					+ H*y[i]*(delta + taxhouse) + taxrate*phi*F ) );
			
		b100n = invhw*( ccons[i] + Cou + beta*Cou*taxrate + beta*ccons[i]*gamma 
				+ H*y[i]*taxhouse + H*y[i]*delta + phi*F*taxrate
				+ beta*(L + xom[i]*r + taxrate*taxhouse*H*y[i] 
				+ gamma*(Cou + H*y[i]*(taxhouse + delta) + F*phi*taxrate)) );
		
		b010p = invhp*( mup*y[i] + beta*gamma*mup*y[i] + 
							(3*beta*pow(y[i],2)*rcconsy[i]*pow(sigmap,2))/2. + 
							pow(beta,2)*gamma*pow(y[i],2)*rcconsy[i]*pow(sigmap,2)); 
		
		b010n = invhp*(	beta*mup*y[i] + pow(y[i],2)*rcconsy[i]*
							pow(sigmap,2) + (pow(beta,2)*pow(y[i],2)*
							rcconsy[i]*pow(sigmap,2))/2  + 
							(3*beta*gamma*pow(y[i],2)*rcconsy[i]*pow(sigmap,2))/2 + 
							(pow(beta,2)*pow(gamma,2)*pow(y[i],2)*rcconsy[i]*
							 pow(sigmap,2))/2 );
					   
		/* Second derivative */
		b200 = 0;
						
		b020 = invhp*invhp*( 0.5*y[i]*y[i]*sigmap*sigmap - 
							    0.5*beta*y[i]*y[i]*sigmap*sigmap +  
								0.5*beta*gamma*y[i]*y[i]*sigmap*sigmap );
		
		b110 = 0;		
        
		if(b110 > 0)
		{
			/* p[x+h,y+g] */
			npxphypg[i] = 0.5*b110;
			/* p[x-h,y-g] */
			npxnhyng[i] = npxphypg[i];
			/* p[x+h,y-g] */
			npxphyng[i] = 0;
			/* p[x-h,y+g] */
			npxnhypg[i] = npxphyng[i];
		}
		else
		{
			/* p[x+h,y+g] */
			npxphypg[i] = 0;
			/* p[x-h,y-g] */
			npxnhyng[i] = npxphypg[i];
			/* p[x+h,y-g] */
			npxphyng[i] = - 0.5*b110;
			/* p[x-h,y+g] */
			npxnhypg[i] = npxphyng[i];
		}			 		
			
		/* Numerator Probabilities */ 
		/* p[x+h,y] */
		npxphy[i] = b100p + b200 - 0.5*abs(b110);        
                /* p[x-h,y] */
		npxnhy[i] = b100n + b200 - 0.5*abs(b110);
		/* p[x,y+g] */
		npxypg[i] = b010p + b020 - 0.5*abs(b110);
		/* p[x,y-g] */
		npxyng[i] = b010n + b020 - 0.5*abs(b110);
		/* Qfixed */ 
		Qf[i] = b001*invtau + b100p + b100n + b010p + b010n + 2*b200 + 2*b020 - abs(b110);
		
		uc[i] = 0;								
		
		invQsi = 1/Qf[i];
		dtxi[i] = invQsi; 
		disci[i] = 1;
						
			if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
			{
				ptau[i] = invQsi*invtau*b001;
				pxypg[i] = invQsi*npxypg[i];
				pxphy[i] = invQsi*npxphy[i];
				pxnhy[i] = invQsi*npxnhy[i];
				pxphypg[i] = invQsi*npxphypg[i];
				pxnhypg[i] = invQsi*npxnhypg[i];
				pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
				pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}
			else if (inx[i] == 1 && iny[i] == 1)																														
			{	
				ptau[i] = invQsi*invtau*b001;
				pxy[i] = 1 - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}	
			else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy)
			{
				ptau[i] = invQsi*invtau*b001;
				pxy[i] = 1 - ptau[i];
            }
			else if (inx[i] == 1 && iny[i] == stepy)
			{
				ptau[i] = invQsi*invtau;
				 pxy[i] = 1 - ptau[i];
				 invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}
			else if (inx[i] > 1 && inx[i] < stepx && iny[i] == stepy)																								
            {	
				ptau[i] = invQsi*invtau*b001;
				pxphy[i] = invQsi*npxphy[i];
				pxnhy[i] = invQsi*npxnhy[i];
				pxy[i] = 1 - pxphy[i] - pxnhy[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
				
			}
			else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
			{
				ptau[i] = invQsi*invtau*b001;
				pxypg[i] = invQsi*npxypg[i];
				pxnhy[i] = invQsi*npxnhy[i];
				pxyng[i] = invQsi*npxyng[i];
				pxnhyng[i] = invQsi*npxnhyng[i];
				pxnhypg[i] = invQsi*npxnhypg[i];
				pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
				pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]); 
			}
			else if (inx[i] == stepx && iny[i] == 1) 
			{
				ptau[i] = invQsi*invtau*b001;
				pxypg[i] = invQsi*npxypg[i];
				pxnhy[i] = invQsi*npxnhy[i];
				pxnhypg[i] = invQsi*npxnhypg[i];
				pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]); 
			}
			else if (inx[i] == 1 && iny[i] == stepy)  
			{
				ptau[i] = invQsi*invtau*b001;
				pxy[i] = 1 - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}	
			else if (inx[i] == stepx && iny[i] == stepy)  
			{
				ptau[i] = invQsi*invtau*b001;
				pxnhy[i] = invQsi*npxnhy[i];
				pxy[i] = 1 - pxnhy[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}
			else
			{
				ptau[i] = invQsi*invtau*b001;
				pxypg[i] = invQsi*npxypg[i];
				pxphy[i] = invQsi*npxphy[i];
				pxnhy[i] = invQsi*npxnhy[i];
				pxyng[i] = invQsi*npxyng[i];
				pxphypg[i] = invQsi*npxphypg[i];
				pxnhyng[i] = invQsi*npxnhyng[i];
				pxnhypg[i] = invQsi*npxnhypg[i];
				pxphyng[i] = invQsi*npxphyng[i];
				pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
				pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
			}
						
		prtau = disci[i]*ptau[i];
        prxy = disci[i]*pxy[i];
		prxphy = disci[i]*pxphy[i];
		prxnhy = disci[i]*pxnhy[i];
		prxypg = disci[i]*pxypg[i];
		prxyng = disci[i]*pxyng[i];
		prxphypg = disci[i]*pxphypg[i];
		prxnhyng = disci[i]*pxnhyng[i];
		prxnhypg = disci[i]*pxnhypg[i];
		prxphyng = disci[i]*pxphyng[i];      
		
			
		felicity =  dtxi[i]*invpr[i]*(  ccons[i]*(- r + theta + rho - theta*
									   pow(ccxss[inx[i]+1][iny[i]+1],-1+beta-beta*gamma)*
												 pow(ccons[i],1-beta+beta*gamma) ) ) ;																			
			
		ccx[inx[i]+1][iny[i]+1] = min_d( felicity + prtau*ccxtime[i] +  prxy*ccxold[inx[i]+1][iny[i]+1] + 
											prxphy*ccxold[inx[i]+2][iny[i]+1] + 
											prxnhy*ccxold[inx[i]][iny[i]+1] +  
											prxypg*ccxold[inx[i]+1][iny[i]+2] + 
											prxyng*ccxold[inx[i]+1][iny[i]] +
											prxphypg*ccxold[inx[i]+2][iny[i]+2] +
											prxnhyng*ccxold[inx[i]][iny[i]] +
											prxnhypg*ccxold[inx[i]][iny[i]+2] +
											prxphyng*ccxold[inx[i]+2][iny[i]], ia[i] );
		}						
	
	}
}


void machinemor(	myvar type, myvar gamma, myvar beta, myvar L, myvar H, myvar r, 
			myvar Cou, myvar taxrate, myvar wmax, myvar invtau,
			   myvar **V, myvar **Vold, myvar *Vtime, myvar **Voldss, myvar theta,
			   int stepx, int stepy, myvar invhw, myvar phi, myvar rho, myvar Mt, myvar Mofree,
			   int *inx, int *inxd, int *iny,
			   myvar *uc, myvar *ia, myvar *ccx,
			   myvar *Qf, myvar *dtxi, myvar *disci,
			   myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
			   myvar *npxypg, myvar *npxyng, 
			   myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
			   myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
			   myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
			   myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prxy;
	myvar prtau;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar felicity;
	myvar mcck = wmax;
	myvar depuc;
	myvar b10pQi;
	myvar b10pi;
	myvar b10nQi;
	myvar b10ni;						
	myvar Qsi;
	myvar invQsi;  
	
	// Step iteration
		for (i = 1; i <= stepx*stepy; i++)
		{	
		        
		        b10pi =  invhw*( L + taxrate*Cou );
			b10pQi =  invhw*( L + taxrate*Cou );				
			b10nQi = invhw*( ccx[i] + taxrate*phi*Mt );
	                b10ni =  invhw*( ccx[i] + taxrate*phi*Mt );						
			
			Qsi = b10pQi + b10nQi + Qf[i];
			invQsi = 1/Qsi;
			dtxi[i] = invQsi;
			disci[i] = exp(-r*dtxi[i]);
			
			if (inxd[i] == 0) 
        	        {
			        ptau[i] = invQsi*invtau;
				pxy[i] = 1 - ptau[i];
				invpr[i] = 1/(1 - disci[i]*pxy[i]);
				prxy = disci[i]*pxy[i];
			       	prtau = disci[i]*ptau[i];
				V[inx[i]+1][iny[i]+1] = prtau*Vtime[i] + prxy*Vold[inx[i]+1][iny[i]+1];
				
			}
    		        else if (inx[i] >= 1 && inx[i] <= stepx && iny[i] == stepy) 
        	        {
				V[inx[i]+1][iny[i]+1] = Mofree;
			}
			else  	
			{	
				uc[i] = Cou;						
				npxnhy[i] = b10ni + nfpxnhy[i];
				npxphy[i] = b10pi + nfpxphy[i];
						
				if (inx[i] == 1 && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxy[i] = 1 - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}				
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxypg[i] = invQsi*npxypg[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 - pxypg[i]  - pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
								pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == 1) 
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxphyng[i] = invQsi*npxphyng[i];
					pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				
			         prxy = disci[i]*pxy[i];
				prtau = disci[i]*ptau[i];
				prxphy = disci[i]*pxphy[i];
				prxnhy = disci[i]*pxnhy[i];
				prxypg = disci[i]*pxypg[i];
				prxyng = disci[i]*pxyng[i];
				prxphypg = disci[i]*pxphypg[i];
				prxnhyng = disci[i]*pxnhyng[i];
				prxnhypg = disci[i]*pxnhypg[i];
				prxphyng = disci[i]*pxphyng[i];      
				myvar mycash = dtxi[i]*(uc[i] + theta*( Voldss[inx[i]+1][iny[i]+1] - Vold[inx[i]+1][iny[i]+1] )) ;
					
			
                                    V[inx[i]+1][iny[i]+1] = mycash + prtau*Vtime[i] + prxy*Vold[inx[i]+1][iny[i]+1] +  
											prxphy*Vold[inx[i]+2][iny[i]+1] + 
											prxnhy*Vold[inx[i]][iny[i]+1] +  
											prxypg*Vold[inx[i]+1][iny[i]+2] + 
											prxyng*Vold[inx[i]+1][iny[i]] +
											prxphypg*Vold[inx[i]+2][iny[i]+2] +
											prxnhyng*Vold[inx[i]][iny[i]] +
											prxnhypg*Vold[inx[i]][iny[i]+2] +
											prxphyng*Vold[inx[i]+2][iny[i]];
			  
			  							
			}	
	}
}


void machinePD(	myvar type, myvar gamma, myvar beta, myvar L, myvar H, myvar r, 
			   myvar Cou, myvar taxrate, myvar wmax, myvar invtau,
			   myvar **V, myvar **Vold, myvar *Vtime, myvar **Voldss, myvar theta,
			   int stepx, int stepy, myvar invhw, myvar phi, myvar rho, myvar Mt, myvar Mofree,
			   int *inx, int *inxd, int *iny,
			   myvar *uc, myvar *ia, myvar *ccx,
			   myvar *Qf, myvar *dtxi, myvar *disci,
			   myvar *nfpxnhy, myvar *nfpxphy, myvar *npxnhy, myvar *npxphy,
			   myvar *npxypg, myvar *npxyng, 
			   myvar *npxphypg, myvar *npxnhyng, myvar *npxnhypg, myvar *npxphyng,
			   myvar *ptau, myvar *pxypg, myvar *pxphy, myvar *pxnhy, myvar *pxyng,
			   myvar *pxphypg, myvar *pxnhyng, myvar *pxnhypg, 
			   myvar *pxphyng, myvar *pxy, myvar *invpr)
{
	
	int it, i;
	myvar prxy;
	myvar prtau;
	myvar prxphy;
	myvar prxnhy;
	myvar prxypg;
	myvar prxyng;
	myvar prxphypg;
	myvar prxnhyng;
	myvar prxnhypg;
	myvar prxphyng;      
	myvar felicity;
	myvar mcck = wmax;
	myvar depuc;
	myvar b10pQi;
	myvar b10pi;
	myvar b10nQi;
	myvar b10ni;						
	myvar Qsi;
	myvar invQsi;  
	
	// Step iteration
		for (i = 1; i <= stepx*stepy; i++)
		{	
		        b10pi =  invhw*( L + taxrate*Cou );
			b10pQi =  invhw*( L + taxrate*Cou );				
			b10nQi = invhw*( ccx[i] + taxrate*phi*Mt );
	                b10ni =  invhw*( ccx[i] + taxrate*phi*Mt );					
			Qsi = b10pQi + b10nQi + Qf[i];
			invQsi = 1/Qsi;
			dtxi[i] = invQsi;
			disci[i] = 1;
			
			if (inxd[i] == 0) 
        	{
				V[inx[i]+1][iny[i]+1] = Vold[inx[i]+1][iny[i]+1];
			}
    		else if (inx[i] >= 1 && inx[i] <= stepx && iny[i] == stepy) 
        	{
				V[inx[i]+1][iny[i]+1] = 0;
			}
			else  	
			{	
				uc[i] = 0;						
				npxnhy[i] = b10ni + nfpxnhy[i];
				npxphy[i] = b10pi + nfpxphy[i];
						
				if (inx[i] == 1 && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxy[i] = 1 - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}				
				else if (inx[i] > 1 && inx[i] < stepx && iny[i] == 1)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxphypg[i] - pxnhypg[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
                                else if (inx[i] == 1 && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxphy[i] = invQsi*npxphy[i];
					pxypg[i] = invQsi*npxypg[i];
					pxyng[i] = invQsi*npxyng[i];
					pxy[i] = 1 - pxypg[i]  - pxphy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] > 1 && iny[i] < stepy)
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhyng[i] - pxnhypg[i] - 
								pxypg[i]  - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else if (inx[i] == stepx && iny[i] == 1) 
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxy[i] = 1 - pxnhypg[i] - pxypg[i] - pxnhy[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]); 
				}
				else
				{
					ptau[i] = invQsi*invtau;
					pxypg[i] = invQsi*npxypg[i];
					pxphy[i] = invQsi*npxphy[i];
					pxnhy[i] = invQsi*npxnhy[i];
					pxyng[i] = invQsi*npxyng[i];
					pxphypg[i] = invQsi*npxphypg[i];
					pxnhyng[i] = invQsi*npxnhyng[i];
					pxnhypg[i] = invQsi*npxnhypg[i];
					pxphyng[i] = invQsi*npxphyng[i];
					pxy[i] = 1 - pxphypg[i] - pxnhyng[i] - pxnhypg[i] - pxphyng[i] -
								pxypg[i] - pxphy[i] - pxnhy[i] - pxyng[i] - ptau[i];
					invpr[i] = 1/(1 - disci[i]*pxy[i]);
				}
				
				prxy = disci[i]*pxy[i];
				prtau = disci[i]*ptau[i];
				prxphy = disci[i]*pxphy[i];
				prxnhy = disci[i]*pxnhy[i];
				prxypg = disci[i]*pxypg[i];
				prxyng = disci[i]*pxyng[i];
				prxphypg = disci[i]*pxphypg[i];
				prxnhyng = disci[i]*pxnhyng[i];
				prxnhypg = disci[i]*pxnhypg[i];
				prxphyng = disci[i]*pxphyng[i];      
				myvar mycash = dtxi[i]*(uc[i] + theta*( Voldss[inx[i]+1][iny[i]+1] - Vold[inx[i]+1][iny[i]+1] )) ;
					
			
                                    V[inx[i]+1][iny[i]+1] = mycash + prtau*Vtime[i] + prxy*Vold[inx[i]+1][iny[i]+1] +  
											prxphy*Vold[inx[i]+2][iny[i]+1] + 
											prxnhy*Vold[inx[i]][iny[i]+1] +  
											prxypg*Vold[inx[i]+1][iny[i]+2] + 
											prxyng*Vold[inx[i]+1][iny[i]] +
											prxphypg*Vold[inx[i]+2][iny[i]+2] +
											prxnhyng*Vold[inx[i]][iny[i]] +
											prxnhypg*Vold[inx[i]][iny[i]+2] +
											prxphyng*Vold[inx[i]+2][iny[i]];						
			}	
	}
}


void defaultind(    int *rowdefault, int *coldefault,
				int **rowdefaultma, int **coldefaultma,
				int *rowrent, int *colrent,
				int **rowrentma, int **colrentma,
				int *rowtimedefault, int *coltimedefault,
				int *ominx, int *ominy, int *ominxd, int *ominxdef, int *rinxdef, int *rinx, int *riny,
				myvar **Vomold, myvar **Vom, myvar **Vr, myvar **ccxom, myvar **ccxr, myvar *Mtime, myvar *collateral,
				myvar *wealth, myvar *xd, myvar *xom, myvar *xr, myvar *y, myvar Mt, myvar H,
				int stepx, int stepy, myvar tresh, myvar wmax, myvar pmax,
				int *nogriddef, int **ominxdma, int *indicati, int *indicatj,
				int **prerowdefaultma, int **precoldefaultma, int *prerowdefault, int *precoldefault,
				int **prerowrentma, int **precolrentma, int *prerowrent, int *precolrent)
{
	int j, i, p, ind, indmap, indj, indp, inddef, def, le, pos, last;
	
	
	myvar diff;
	myvar checkvm1;
	
  	/* Default index */
	for(j = 1; j <= stepy; j++)
	{
		indmap = 1 + (j-1)*stepx;
		
		if (xd[indmap] < xr[indmap] )
		{
			/*printf("Regime First \n");*/
			
			for(i = 1; i <= stepx ; i++)
			{
				ind = i + (j-1)*stepx;
				
				for (p = 1; p <= stepx; p++)
				{
					indp = p + (j-1)*stepx;
					diff = xd[indp] - xr[ind];
					
					if ( abs_d(diff) < tresh && xd[indp] > 0 )
					{	
						ominxdef[ind] = p;
						rinxdef[ind] = i;
						nogriddef[ind] = 0;
						break;
					}
				}
				
				if ( i > 1 )
				{
					if ( ominxdef[ind] == 0 && ominxdef[ind - 1] == stepx && i <= stepx)
					{
						ominxdef[ind] = stepx;
					}
					else if ( ominxdef[ind] == ominxdef[ind - 1] &&  ominxdef[ind] < stepx && i <= stepx)
					{
						ominxdef[ind] = ominxdef[ind - 1] + 1;
					}
					else if ( ominxdef[ind] < ominxdef[ind - 1] &&  ominxdef[ind] < stepx && ominy[ind] == ominy[ind-1])
					{
						ominxdef[ind] = ominxdef[ind - 1] + 1;
					}
				}
				
				if ( i > 1 )
				{
					if ( rinxdef[ind] == 0 && rinxdef[ind - 1] > 0 && i <= stepx)
					{
						rinxdef[ind] = rinxdef[ind - 1] + 1;
					}
				}
			}
		}
		else
		{
			/*printf("Regime Second \n");*/
			
			for (p = 1; p <= stepx; p++)
			{
				indp = p + (j-1)*stepx;
				
				for(i = 1; i <= stepx ; i++)
				{
					ind = i + (j-1)*stepx;
					diff = xd[indp] - xr[ind];
					
					if ( abs_d(diff) < tresh && xd[indp] > 0 && xd[indp] < wmax + H*pmax)
					{	
						ominxdef[indp] = p;
						rinxdef[indp] = i;
						nogriddef[indp] = 0;
						break;
					}
				}
				
				if ( i > 1 )
				{
					if ( rinxdef[indp] == 0 && rinxdef[indp - 1] == stepx && i <= stepx)
					{
						rinxdef[indp] = stepx;
					}
					else if ( rinxdef[indp] == rinxdef[indp - 1] &&  rinxdef[indp] < stepx && i <= stepx)
					{
						rinxdef[indp] = rinxdef[indp - 1] + 1;
					}
					else if ( rinxdef[indp] < rinxdef[indp - 1] &&  rinxdef[indp] < stepx && ominy[indp] == ominy[indp-1])
					{
						rinxdef[indp] = rinxdef[indp - 1] + 1;
					}
				}
			}
		}
	}
	
	
	for(j = 1;j <= stepy; j++)
	{
		for (i = 1; i <= stepx; i++)
		{
			if ( coldefaultma[i][j] > 0 )
			{
				indicatj[j] = coldefaultma[i][j];
				break;
			}
		}
	}	
	
	for(j = 1;j <= stepy; j++)
	{
		indicati[j] = 1;
	}
	
	for(j = 1;j <= stepy; j++)
	{
		for (i = stepx; i >= 1; i--)
		{
			if ( rowdefaultma[i][j] > 0 )
			{
				indicati[j] = rowdefaultma[i][j];
				break;
			}
		}
	}
	
	def = 0;
	for (j = 1; j <= stepy; j++) 
	{					
		
		for (i = 1; i <= stepx; i++) 
		{
			ind = i + (j-1)*stepx;
			inddef = ominxdef[ind] + (j-1)*stepx;
			
			if ( nogriddef[ind] == 0 )
			{		
				checkvm1 = Vom[ominxdef[ind]+1][ominy[ind]+1] - Vr[rinxdef[ind]+1][riny[ind]+1];
				
				if ( checkvm1 <= 0 && ominxd[inddef] > 0 ) 
				{		
				  	if ( ominy[ind] > indicatj[j-1] + 1 )
					{
						break;
					}
					else
					{
						prerowdefault[inddef] = ominxdef[ind];
						precoldefault[inddef] = ominy[ind];
						prerowrent[inddef] = rinxdef[ind];
						precolrent[inddef] = riny[ind];
						
						ominxd[inddef] = 1000;
						indicati[j] = rinxdef[ind];
						def = 1;
					}
				}
			}
		}
	}
    
	
	if (def == 1)
	{	
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				prerowdefaultma[i][j] = prerowdefault[ind];
				precoldefaultma[i][j] = precoldefault[ind];
				prerowrentma[i][j] = prerowrent[ind];
				precolrentma[i][j] = precolrent[ind];
			}
		}
		
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				if ( ominxd[ind] > 0 )
				{
					ominxdma[i][j] = ominxd[ind];
				}	
			}
		}
		
		for( j = 1; j <= stepy-1; j++)
		{
			for (i = stepx; i >= 1; i--)
			{
				if ( prerowdefaultma[i][j] > prerowdefaultma[i-1][j] && prerowdefaultma[i-1][j] == 0 )
				{
					prerowdefaultma[i-1][j] = prerowdefaultma[i][j]-1;
					precoldefaultma[i-1][j] = precoldefaultma[i][j];
				}
			}    
		}
	
		for(j = 1; j <= stepy-1; j++)
		{
			for (i = stepx; i >= 1; i--)
			{
				if ( prerowrentma[i][j] > prerowrentma[i-1][j] && prerowrentma[i-1][j] == 0 )
				{
					prerowrentma[i-1][j] = prerowrentma[i][j]-1;
					precolrentma[i-1][j] = precolrentma[i][j];
				}
			}    
		}
						
		for(j = 1; j <= stepy-1; j++)
		{
			for (i = stepx; i > 1; i--)
			{
				if ( prerowdefaultma[i][j] > 0  )
				{
					for (p = prerowdefaultma[i][j]; p > 0; p-- )
					{
						if ( ominxdma[p][j] > 0 )
						{
							ominxdma[p][j] = 1000;
						}	
					}
					break;
				}
			}    
		}
		
		
		for(j = stepy; j > 1; j--)
		{
			for (i = stepx; i > 0; i--)
			{
				if ( prerowdefaultma[i][j] > rowdefaultma[i][j-1] && rowdefaultma[i][j-1] == 0 )
				{
					prerowdefaultma[i][j-1] = prerowdefaultma[i][j];
					prerowrentma[i][j-1] = prerowrentma[i][j];
					precoldefaultma[i][j-1] = precoldefaultma[i][j]-1;
					precolrentma[i][j-1] = precolrentma[i][j]-1;
					ominxdma[i][j-1] = 1000;
					
				}
			}    
		}
		
		
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				prerowdefault[ind] = prerowdefaultma[i][j];
				precoldefault[ind] = precoldefaultma[i][j];
			}
		}
		
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				prerowrent[ind] = prerowrentma[i][j];
				precolrent[ind] = precolrentma[i][j];
			}
		}
		
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				if ( ominxd[ind] > 0 )
				{
					ominxd[ind] = ominxdma[i][j];
				}	
			}
		}
		
		for ( i = 1 ; i <= stepx*stepy; i++ )					
		{				
			if (prerowdefault[i] == 1 && prerowrent[i] > 0 && ominxd[i] == 1000 )
			{
				Vom[prerowdefault[i]+1][precoldefault[i]+1] = Vr[prerowrent[i]+1][precolrent[i]+1];
			        Mtime[i] = collateral[i];
				ccxom[prerowdefault[i]+1][precoldefault[i]+1] = 0.5;
				rowdefault[i] = prerowdefault[i];
				coldefault[i] = precoldefault[i];
				rowrent[i] = prerowrent[i];
				colrent[i] = precolrent[i];
				ominxd[i] = 0;			
			}
			else if (prerowdefault[i] > 1 && prerowrent[i] > 0 && ominxd[i] == 1000 )
			{
				Vom[prerowdefault[i]+1][precoldefault[i]+1] = Vr[prerowrent[i]+1][precolrent[i]+1];
				Mtime[i] = collateral[i];					
				ccxom[prerowdefault[i]+1][precoldefault[i]+1] = ccxr[prerowrent[i]+1][precolrent[i]+1];
				rowdefault[i] = prerowdefault[i];
				coldefault[i] = precoldefault[i];
				rowrent[i] = prerowrent[i];
				colrent[i] = precolrent[i];
				ominxd[i] = 0;			
			}
			else if (prerowdefault[i] > 0 && prerowrent[i] <= 0 && ominxd[i] == 1000 )
			{
				Vom[prerowdefault[i]+1][precoldefault[i]+1] = Vom[prerowdefault[i]+1][precoldefault[i]+1];
				Mtime[i] = collateral[i];
				ccxom[prerowdefault[i]+1][precoldefault[i]+1] = ccxom[prerowdefault[i]+1][precoldefault[i]+1];
				rowdefault[i] = prerowdefault[i];
				coldefault[i] = precoldefault[i];
				rowrent[i] = prerowrent[i];
				colrent[i] = precolrent[i];
				ominxd[i] = 0;			
			}
			else
			{
				continue;
			}
		}
		
		for(j = 1;j <= stepy; j++)
		{
			for(i = 1;i <= stepx; i++)
			{
				ind = i + (j-1)*stepx;
				rowdefaultma[i][j] = rowdefault[ind];
				coldefaultma[i][j] = coldefault[ind];
				rowrentma[i][j] = rowrent[ind];
				colrentma[i][j] = colrent[ind];
			}
		}
	}				
}

int main()
{
    
	int i, j, it, time, itime, ilambda, ind, indd, inddef;
	
    myvar age = 25.0;			/* Householdhead's age              */
	myvar T = 30.0;				/* Time horizon                     */
	myvar Tret = 15.0;	
    myvar gamma = 2;			/* Relative risk aversion						*/
	myvar beta = 0.7;			/* Cobb-Douglas - Housing allocation			*/
	myvar rho = 0.03;			/* Rate of time preference						*/
	myvar r = 0.03;				/* Lending risk free rate						*/
	myvar mup = 0.016;			/* House price drift							*/
  	myvar sigmap = 0.1;			/* House price volatility						*/
    myvar scalebound = 1;
	myvar H = 180;				/* House size									*/
	myvar LG0 = 36.8053;				/* Labor income - Good state					*/
	myvar muL0 = - 4.029590;
	myvar muL1 = 0.278772;
	myvar muL2 = - 0.053005;
	myvar muL3 = 0.003097;
	myvar LBfrac = 0.559700;				/* Labor income	- Bad state				*/       
	myvar thetaG = 0.0742;			/* Good State									*/
	myvar thetaB = 0.4467; 			/* Bad State	*/
	myvar kappa = 0.07;			/* Down payment									*/
	myvar dead = 1 - 0.4;                        /* Deadweight losses */
	myvar epsilon = 0.15;
	myvar taxrate = 0.2;
	myvar taxhouse = 0.015;
	myvar delta = 0.022;
	int type;				/* Owner = 1; Renter = 2; Mortgage = 3 */
	
	/* Parameters Loop   */
       
    myvar lambda = 0.0124;
    myvar spread = 0.0011;
    myvar posxr = LG0;
	myvar posxom = posxr - kappa*H;
	myvar posy = 1.01;
	int posgridxom;
	int posgridxr;
	int posgridy;

	/* Define some parameters */
	myvar alpha = r - mup + lambda + taxhouse + delta;
	myvar Mo = (1 - kappa)*H;
	myvar R = r + spread;	
    myvar Cou = R*Mo/(1 - exp(- R*T));
	myvar Courf = r*Mo/(1 - exp(- r*T));
	myvar focc = 1/(beta*(1-gamma) - 1);
    myvar ratgamma = 1/(1 - gamma);
	myvar invgamma = 1/gamma;
	myvar invbeta = 1/beta;
	myvar bega = - (1 - beta)*(1 - gamma);
	myvar focH = pow(H,bega);

	myvar fac1 = pow(beta,beta)*pow(1-beta,1-beta);
	myvar factor = pow(fac1,1-gamma)/(1-gamma);
	myvar intdiscount = (1 - exp( - rho*Tret))/rho;
	myvar annuity = r*pow(1+r,Tret)/(pow(1+r,Tret)-1); 

	myvar tau = 0.005;
	myvar stept = T/tau;			/* Time steps */
	myvar invtau = 1/tau;
	myvar stept2 = 12*T;
	myvar tau2 = T/stept2;
	myvar invtau2 = 1/tau2;

    myvar *parameters;
	parameters = (myvar*) malloc(sizeof(myvar) * (25));
	parameters[1] = age;		
	parameters[2] = T;			
	parameters[3] = gamma;		
	parameters[4] = beta;	
	parameters[5] = rho;
	parameters[6] = r;	
	parameters[7] = mup;			
  	parameters[8] = sigmap;		
    parameters[9] = H;			
	parameters[10] = LG0;		
	parameters[11] = muL0;
	parameters[12] = muL1;
	parameters[13] = muL2;
	parameters[14] = muL3;
	parameters[15] = LBfrac;		       
	parameters[16] = thetaG;	
	parameters[17] = thetaB; 
	parameters[18] = kappa;		
	parameters[19] = dead;                    
	parameters[20] = epsilon;
	parameters[21] = taxrate;
	parameters[22] = taxhouse;
	parameters[23] = delta;
	parameters[24] = spread;
	parameters[25] = lambda;

	//print variables
    #define PARAM_FILE						"Parameters.dat"	
    #define VRG_FILE						"VrG.dat"
	#define VOMG_FILE						"VomG.dat"
	#define CCXOMG_FILE						"ccxomG.dat"
	#define VMORG_FILE						"VmorG.dat"
	#define COLMAG_FILE						"colmaG.dat"
	#define ROWMAG_FILE						"rowmaG.dat"
	#define VRB_FILE						"VrB.dat"
	#define VOMB_FILE						"VomB.dat"
	#define CCXOMB_FILE						"ccxomB.dat"
	#define VMORB_FILE						"VmorB.dat"
	#define COLMAB_FILE						"colmaB.dat"
	#define ROWMAB_FILE						"rowmaB.dat"
	#define OMINXDEFG_FILE					"ominxdefG.dat"
	#define OMINXDEFB_FILE					"ominxdefB.dat"
	#define RINXDEFG_FILE					"rinxdefG.dat"
	#define RINXDEFB_FILE					"rinxdefB.dat"
	#define OMINXDG_FILE					"ominxdG.dat"
	#define OMINXDB_FILE					"ominxdB.dat"
	#define XD_FILE							"xd.dat"
	#define XOM_FILE						"xom.dat"
	#define XR_FILE							"xr.dat"
	#define Y_FILE							"y.dat"
	
    #define PDOMG_FILE						"PDomG.dat"
	#define PDOMB_FILE						"PDomB.dat"
		
	#define StorevaluesccxomG_FILE			"StorevaluesccxomG.dat"
	#define StorevaluesccxomB_FILE			"StorevaluesccxomB.dat"
	#define StorevaluesccxrG_FILE			"StorevaluesccxrG.dat"
	#define StorevaluesccxrB_FILE			"StorevaluesccxrB.dat"
	#define StorevaluesominxdG_FILE			"StorevaluesominxdG.dat"
	#define StorevaluesominxdB_FILE			"StorevaluesominxdB.dat"
	#define StorevaluesominxdefG_FILE		"StorevaluesominxdefG.dat"
	#define StorevaluesominxdefB_FILE		"StorevaluesominxdefB.dat"
	#define StorevaluesrinxdefG_FILE		"StorevaluesrinxdefG.dat"
	#define StorevaluesrinxdefB_FILE		"StorevaluesrinxdefB.dat"
	
	FILE *fp;

/* 
* Algorithm parameters
*/

	int imax = 500;				/* Max number iterations */
	myvar tol = 1e-005;			/* Tolerance coefficient  */
	int stepx = MYSTEPX;		/* Number of steps */
	int stepy = MYSTEPY;		/* Number of steps */
	myvar changestart = 0;		/* Inizialization norm calculation */
	myvar changeG;
	myvar changeB;
	myvar temp;
	
	// Grid 
	myvar wmax = 250;
	myvar wmin = 0.01;
	myvar hw = (wmax - wmin)/stepx; 
	myvar pmax = 2;
	myvar pmin = 0.01;
	myvar tresh = hw/2;
	myvar hp = (pmax - pmin)/stepy;
	myvar invhw = 1/hw;
	myvar invhp = 1/hp;

        myvar *LG;
	myvar *LB;
	LG = (myvar*) malloc(sizeof(myvar) * (stept));
	LB = (myvar*) malloc(sizeof(myvar) * (stept));
	
	myvar *LGav;
	myvar *LBav;
	LGav = (myvar*) malloc(sizeof(myvar) * (stept));
	LBav = (myvar*) malloc(sizeof(myvar) * (stept));

	for (time = stept-1; time >= 1 ; time-- ) 
	{ 
	     LG[time] = (1 - taxrate)*LG0*exp( muL0 + muL1*(time*tau + age) + muL2*( pow(time*tau+age,2)/10 ) + muL3*( pow(time*tau+age,3)/100 ) );
	     LB[time] = LG[time]*LBfrac;
	}

	LGav[1] = LG[1];

	for (time = 2; time <= stept-1; time++ ) 
	{ 
	  LGav[time] = (LG[time] + LGav[time-1])/time;
	  LBav[time] = LGav[time]*LBfrac;
	}

        myvar LGaverage = 58.075;
	myvar LBaverage =  LGaverage*LBfrac;

	/* Probability Default */ 
	/* Value function */
	myvar *PDomtimeG;
	myvar *PDomtimeoldG;
	myvar **PDomG;
	myvar **PDomoldG;
	/* Balance */
	myvar *PDiaomG;
	/* Markov probabilities - vectors */
	myvar *PDfelicityvG;		
	myvar *PDucomvG;		
	myvar *PDdtxiomvG;
	myvar *PDdisciomvG;
	myvar *PDptauomvG;
	myvar *PDpxphyomvG;
	myvar *PDpxnhyomvG;
	myvar *PDpxypgomvG;
	myvar *PDpxyngomvG;	
	myvar *PDpxphypgomvG;
	myvar *PDpxnhyngomvG;
	myvar *PDpxphyngomvG;
	myvar *PDpxnhypgomvG;
	myvar *PDpxyomvG;
	myvar *PDinvpromvG;
	myvar *PDnpxphyomvG;
	myvar *PDnpxnhyomvG;
	myvar *PDnpxypgomvG;
	myvar *PDnpxyngomvG;
	myvar *PDnfpxphyomvG;
	myvar *PDnfpxnhyomvG;
	myvar *PDnpxphypgomvG;       
	myvar *PDnpxnhyngomvG;
	myvar *PDnpxphyngomvG;
	myvar *PDnpxnhypgomvG;
	myvar *PDQfomvG;
	
	PDomG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	PDomoldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		PDomG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		PDomoldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	PDomtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDomtimeoldG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDiaomG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDfelicityvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	PDucomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	PDdtxiomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	PDdisciomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDptauomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    PDinvpromvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnfpxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnfpxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	PDnpxnhyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxnhypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDQfomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

/* Value function */
	myvar *PDomtimeB;
	myvar *PDomtimeoldB;
	myvar **PDomB;
	myvar **PDomoldB;
	/* Balance */
	myvar *PDiaomB;
	/* Markov probabilities - vectors */
	myvar *PDfelicityvB;		
	myvar *PDucomvB;		
	myvar *PDdtxiomvB;
	myvar *PDdisciomvB;
	myvar *PDptauomvB;
	myvar *PDpxphyomvB;
	myvar *PDpxnhyomvB;
	myvar *PDpxypgomvB;
	myvar *PDpxyngomvB;	
	myvar *PDpxphypgomvB;
	myvar *PDpxnhyngomvB;
	myvar *PDpxphyngomvB;
	myvar *PDpxnhypgomvB;
	myvar *PDpxyomvB;
	myvar *PDinvpromvB;
	myvar *PDnpxphyomvB;
	myvar *PDnpxnhyomvB;
	myvar *PDnpxypgomvB;
	myvar *PDnpxyngomvB;
	myvar *PDnfpxphyomvB;
	myvar *PDnfpxnhyomvB;
	myvar *PDnpxphypgomvB;       
	myvar *PDnpxnhyngomvB;
	myvar *PDnpxphyngomvB;
	myvar *PDnpxnhypgomvB;
	myvar *PDQfomvB;
	
	PDomB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	PDomoldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		PDomB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		PDomoldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	PDomtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDomtimeoldB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDiaomB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDucomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDfelicityvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDdtxiomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	PDdisciomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDptauomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxphyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxnhypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDpxyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    PDinvpromvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnfpxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnfpxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	PDnpxnhyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxphyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDnpxnhypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	PDQfomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

/* 
* Variables Owning
*/
	/* Value function */
	myvar *VomtimeG;
	myvar *VomtimeoldG;
	myvar **VomG;
	myvar **VomoldG;
	/* Balance */
	myvar *iaomG;
	/* Markov probabilities - vectors */
	myvar *felicityvG;		
	myvar *ucomvG;		
	myvar *dtxiomvG;
	myvar *disciomvG;
	myvar *ptauomvG;
	myvar *pxphyomvG;
	myvar *pxnhyomvG;
	myvar *pxypgomvG;
	myvar *pxyngomvG;	
	myvar *pxphypgomvG;
	myvar *pxnhyngomvG;
	myvar *pxphyngomvG;
	myvar *pxnhypgomvG;
	myvar *pxyomvG;
	myvar *invpromvG;
	myvar *npxphyomvG;
	myvar *npxnhyomvG;
	myvar *npxypgomvG;
	myvar *npxyngomvG;
	myvar *nfpxphyomvG;
	myvar *nfpxnhyomvG;
	myvar *npxphypgomvG;       
	myvar *npxnhyngomvG;
	myvar *npxphyngomvG;
	myvar *npxnhypgomvG;
	myvar *QfomvG;
	
	VomG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	VomoldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		VomG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		VomoldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	VomtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	VomtimeoldG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	iaomG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	felicityvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	ucomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxiomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	disciomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptauomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invpromvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfomvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

/* Value function */
	myvar *VomtimeB;
	myvar *VomtimeoldB;
	myvar **VomB;
	myvar **VomoldB;
	/* Balance */
	myvar *iaomB;
	/* Markov probabilities - vectors */
	myvar *felicityvB;		
	myvar *ucomvB;		
	myvar *dtxiomvB;
	myvar *disciomvB;
	myvar *ptauomvB;
	myvar *pxphyomvB;
	myvar *pxnhyomvB;
	myvar *pxypgomvB;
	myvar *pxyngomvB;	
	myvar *pxphypgomvB;
	myvar *pxnhyngomvB;
	myvar *pxphyngomvB;
	myvar *pxnhypgomvB;
	myvar *pxyomvB;
	myvar *invpromvB;
	myvar *npxphyomvB;
	myvar *npxnhyomvB;
	myvar *npxypgomvB;
	myvar *npxyngomvB;
	myvar *nfpxphyomvB;
	myvar *nfpxnhyomvB;
	myvar *npxphypgomvB;       
	myvar *npxnhyngomvB;
	myvar *npxphyngomvB;
	myvar *npxnhypgomvB;
	myvar *QfomvB;
	
	VomB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	VomoldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		VomB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		VomoldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	VomtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	VomtimeoldB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	iaomB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ucomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	felicityvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dtxiomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	disciomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptauomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invpromvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfomvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
	/* Value function */
	myvar *ccxomtimeG;
	myvar *dccxomxG;
	myvar *dccxomyG;
	myvar *rcconsxG;
	myvar *rcconsyG;
	myvar **ccxomG;
	myvar **ccxomoldG;
	myvar *cconsomG;
	myvar *cfuckomG;
	/* Markov probabilities - vectors */
	myvar *ucomcG;		
	myvar *dtxiomcG;
	myvar *disciomcG;
	myvar *ptauomcG;
	myvar *pxphyomcG;
	myvar *pxnhyomcG;
	myvar *pxypgomcG;
	myvar *pxyngomcG;	
	myvar *pxphypgomcG;
	myvar *pxnhyngomcG;
	myvar *pxphyngomcG;
	myvar *pxnhypgomcG;
	myvar *pxyomcG;
	myvar *invpromcG;
	myvar *npxphyomcG;
	myvar *npxnhyomcG;
	myvar *npxypgomcG;
	myvar *npxyngomcG;
	myvar *nfpxphyomcG;
	myvar *nfpxnhyomcG;
	myvar *npxphypgomcG;       
	myvar *npxnhyngomcG;
	myvar *npxphyngomcG;
	myvar *npxnhypgomcG;
	myvar *QfomcG;
	
	ccxomG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	ccxomoldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ccxomG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		ccxomoldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	ccxomtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxomxG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxomyG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rcconsxG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rcconsyG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cfuckomG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cconsomG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	ucomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxiomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	disciomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptauomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invpromcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfomcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
	/* Value function */
	myvar *ccxomtimeB;
	myvar *dccxomxB;
	myvar *dccxomyB;
	myvar *rcconsxB;
	myvar *rcconsyB;
	myvar **ccxomB;
	myvar **ccxomoldB;
	myvar *cconsomB;
	myvar *cfuckomB;
	/* Markov probabilities - vectors */
	myvar *ucomcB;		
	myvar *dtxiomcB;
	myvar *disciomcB;
	myvar *ptauomcB;
	myvar *pxphyomcB;
	myvar *pxnhyomcB;
	myvar *pxypgomcB;
	myvar *pxyngomcB;	
	myvar *pxphypgomcB;
	myvar *pxnhyngomcB;
	myvar *pxphyngomcB;
	myvar *pxnhypgomcB;
	myvar *pxyomcB;
	myvar *invpromcB;
	myvar *npxphyomcB;
	myvar *npxnhyomcB;
	myvar *npxypgomcB;
	myvar *npxyngomcB;
	myvar *nfpxphyomcB;
	myvar *nfpxnhyomcB;
	myvar *npxphypgomcB;       
	myvar *npxnhyngomcB;
	myvar *npxphyngomcB;
	myvar *npxnhypgomcB;
	myvar *QfomcB;
	
	ccxomB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	ccxomoldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ccxomB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		ccxomoldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	ccxomtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxomxB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxomyB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rcconsxB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rcconsyB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

	cfuckomB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cconsomB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	ucomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxiomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	disciomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptauomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invpromcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfomcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
/* 
* Variables Renting - Good State
*/
	/* Value function */
	myvar *VrtimeG;
	myvar **VrG;
	myvar **VroldG;
	/* Balance */
	myvar *iarG;
	/* Markov probabilities - vectors */
	myvar *ucrvG;		
	myvar *dtxirvG;
	myvar *discirvG;
	myvar *ptaurvG;
	myvar *pxphyrvG;
	myvar *pxnhyrvG;
	myvar *pxypgrvG;
	myvar *pxyngrvG;	
	myvar *pxphypgrvG;
	myvar *pxnhyngrvG;
	myvar *pxphyngrvG;
	myvar *pxnhypgrvG;
	myvar *pxyrvG;
	myvar *invprrvG;
	myvar *npxphyrvG;
	myvar *npxnhyrvG;
	myvar *npxypgrvG;
	myvar *npxyngrvG;
	myvar *nfpxphyrvG;
	myvar *nfpxnhyrvG;
	myvar *npxphypgrvG;       
	myvar *npxnhyngrvG;
	myvar *npxphyngrvG;
	myvar *npxnhypgrvG;
	myvar *QfrvG;
	
	VrG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	VroldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		VrG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		VroldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	VrtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	iarG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ucrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxirvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	discirvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaurvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invprrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfrvG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

/* Value function */
	myvar *VrtimeB;
	myvar **VrB;
	myvar **VroldB;
	/* Balance */
	myvar *iarB;
	/* Markov probabilities - vectors */
	myvar *ucrvB;		
	myvar *dtxirvB;
	myvar *discirvB;
	myvar *ptaurvB;
	myvar *pxphyrvB;
	myvar *pxnhyrvB;
	myvar *pxypgrvB;
	myvar *pxyngrvB;	
	myvar *pxphypgrvB;
	myvar *pxnhyngrvB;
	myvar *pxphyngrvB;
	myvar *pxnhypgrvB;
	myvar *pxyrvB;
	myvar *invprrvB;
	myvar *npxphyrvB;
	myvar *npxnhyrvB;
	myvar *npxypgrvB;
	myvar *npxyngrvB;
	myvar *nfpxphyrvB;
	myvar *nfpxnhyrvB;
	myvar *npxphypgrvB;       
	myvar *npxnhyngrvB;
	myvar *npxphyngrvB;
	myvar *npxnhypgrvB;
	myvar *QfrvB;
	
	VrB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	VroldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		VrB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		VroldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	VrtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	iarB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ucrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxirvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	discirvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaurvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invprrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfrvB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
	/* Value function */
	myvar *ccxrtimeG;
	myvar *cfacrG;
	myvar *dccxrxG;
	myvar *dccxryG;
	myvar **ccxrG;
	myvar **ccxroldG;
	myvar *cconsrG;
	myvar *cfuckrG;
	/* Markov probabilities - vectors */
	myvar *ucrcG;		
	myvar *dtxircG;
	myvar *discircG;
	myvar *ptaurcG;
	myvar *pxphyrcG;
	myvar *pxnhyrcG;
	myvar *pxypgrcG;
	myvar *pxyngrcG;	
	myvar *pxphypgrcG;
	myvar *pxnhyngrcG;
	myvar *pxphyngrcG;
	myvar *pxnhypgrcG;
	myvar *pxyrcG;
	myvar *invprrcG;
	myvar *npxphyrcG;
	myvar *npxnhyrcG;
	myvar *npxypgrcG;
	myvar *npxyngrcG;
	myvar *nfpxphyrcG;
	myvar *nfpxnhyrcG;
	myvar *npxphypgrcG;       
	myvar *npxnhyngrcG;
	myvar *npxphyngrcG;
	myvar *npxnhypgrcG;
	myvar *QfrcG;
	
	ccxrG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	ccxroldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ccxrG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		ccxroldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	ccxrtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cfacrG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxrxG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxryG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
	cfuckrG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cconsrG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	ucrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxircG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	discircG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaurcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invprrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfrcG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	
	/* Value function */
	myvar *ccxrtimeB;
	myvar *cfacrB;
	myvar *dccxrxB;
	myvar *dccxryB;
	myvar **ccxrB;
	myvar **ccxroldB;
	myvar *cconsrB;
	myvar *cfuckrB;
	/* Markov probabilities - vectors */
	myvar *ucrcB;		
	myvar *dtxircB;
	myvar *discircB;
	myvar *ptaurcB;
	myvar *pxphyrcB;
	myvar *pxnhyrcB;
	myvar *pxypgrcB;
	myvar *pxyngrcB;	
	myvar *pxphypgrcB;
	myvar *pxnhyngrcB;
	myvar *pxphyngrcB;
	myvar *pxnhypgrcB;
	myvar *pxyrcB;
	myvar *invprrcB;
	myvar *npxphyrcB;
	myvar *npxnhyrcB;
	myvar *npxypgrcB;
	myvar *npxyngrcB;
	myvar *nfpxphyrcB;
	myvar *nfpxnhyrcB;
	myvar *npxphypgrcB;       
	myvar *npxnhyngrcB;
	myvar *npxphyngrcB;
	myvar *npxnhypgrcB;
	myvar *QfrcB;
	
	ccxrB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	ccxroldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ccxrB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		ccxroldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	ccxrtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cfacrB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxrxB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dccxryB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cfuckrB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	cconsrB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	ucrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	dtxircB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy)); 
	discircB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaurcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
    invprrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhyrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfrcB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));

	
/* 
* Variables Mortgage - Good State
*/
	/* Value function */
	myvar *MtimeG;
	myvar **MG;
	myvar **MoldG;
    /* Markov probabilities - vectors */
	myvar *div;		
	myvar *ucmG;
	myvar *dtximG;
	myvar *discimG;
	myvar *ptaumG;
	myvar *pxypgmG;
	myvar *pxphymG;
	myvar *pxnhymG;
	myvar *pxyngmG;
	myvar *pxphypgmG;
	myvar *pxnhyngmG;
	myvar *pxphyngmG;
	myvar *pxnhypgmG;
	myvar *pxymG;
	myvar *invprmG;
	myvar *npxphymG;
	myvar *npxnhymG;
	myvar *npxypgmG;
	myvar *npxyngmG;
	myvar *nfpxphymG;
	myvar *nfpxnhymG;
	myvar *nfpxypgmG;
	myvar *nfpxyngmG;
	myvar *npxphypgmG;       
	myvar *npxnhyngmG;
	myvar *npxphyngmG;
	myvar *npxnhypgmG;
	myvar *QfmG;
	myvar *collateral;
	
	/* Default Information */
	int *rowdefaultG;
	int *coldefaultG;
	int *rowrentG;
	int *colrentG;
	
	int **rowdefaultmaG;
	int **coldefaultmaG;
	int **rowrentmaG;
	int **colrentmaG;

	int *rowtimedefaultG;
	int *coltimedefaultG;
	
	myvar **DefaultboundG;
	myvar **DefaultboundB;
	
	MG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	MoldG = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	rowdefaultmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	coldefaultmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	rowrentmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	colrentmaG = (int**) malloc(sizeof(int*) * (stepx+2));

	for(i=0; i<= (stepx+2); i++)
	{
		MG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		MoldG[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}
	
	for(i=0; i<= (stepx+2); i++)
	{
		rowdefaultmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
		coldefaultmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	for(i=0; i<= (stepx+2); i++)
	{
		rowrentmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
		colrentmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	myvar *ProbdefG;
	ProbdefG = (myvar*) malloc(sizeof(myvar) * (stept));
	
	myvar *DefaultboundaryG;
	DefaultboundaryG = (myvar*) malloc(sizeof(myvar) * (6*stepy*stept));
	
	myvar *DefaultboundaryB;
	DefaultboundaryB = (myvar*) malloc(sizeof(myvar) * (6*stepy*stept));	
	
	MtimeG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	div = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	ucmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dtximG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	discimG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaumG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	invprmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhymG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfmG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rowdefaultG = (int*) malloc(sizeof(int) * (stepx*stepy));
	coldefaultG = (int*) malloc(sizeof(int) * (stepx*stepy));
	rowrentG = (int*) malloc(sizeof(int) * (stepx*stepy));
	colrentG = (int*) malloc(sizeof(int) * (stepx*stepy));
	collateral = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rowtimedefaultG = (int*) malloc(sizeof(int) * (stepx+2));
	coltimedefaultG = (int*) malloc(sizeof(int) * (stepx+2));

/* 
* Variables Mortgage - Bad State
*/
	/* Value function */
	myvar *MtimeB;
	myvar **MB;
	myvar **MoldB;
    /* Markov probabilities - vectors */
	myvar *ucmB;
	myvar *dtximB;
	myvar *discimB;
	myvar *ptaumB;
	myvar *pxypgmB;
	myvar *pxphymB;
	myvar *pxnhymB;
	myvar *pxyngmB;
	myvar *pxphypgmB;
	myvar *pxnhyngmB;
	myvar *pxphyngmB;
	myvar *pxnhypgmB;
	myvar *pxymB;
	myvar *invprmB;
	myvar *npxphymB;
	myvar *npxnhymB;
	myvar *npxypgmB;
	myvar *npxyngmB;
	myvar *nfpxphymB;
	myvar *nfpxnhymB;
	myvar *nfpxypgmB;
	myvar *nfpxyngmB;
	myvar *npxphypgmB;       
	myvar *npxnhyngmB;
	myvar *npxphyngmB;
	myvar *npxnhypgmB;
	myvar *QfmB;
	
	/* Default Information */
	int *rowdefaultB;
	int *coldefaultB;
	int *rowrentB;
	int *colrentB;

	int **rowdefaultmaB;
	int **coldefaultmaB;
	int **rowrentmaB;
	int **colrentmaB;

	int *rowtimedefaultB;
	int *coltimedefaultB;
		
	MB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	MoldB = (myvar**) malloc(sizeof(myvar*) * (stepx+2));
	rowdefaultmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	coldefaultmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	rowrentmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	colrentmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	
	for(i=0; i<= (stepx+2); i++)
	{
		MB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
		MoldB[i] = (myvar*)malloc(sizeof(myvar) * (stepx+2) );
	}

	for(i=0; i<= (stepx+2); i++)
	{
		rowdefaultmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
		coldefaultmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	for(i=0; i<= (stepx+2); i++)
	{
		rowrentmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
		colrentmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	MtimeB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	div = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	ucmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	dtximB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	discimB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	ptaumB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxphyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxnhypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	pxymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	invprmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxphymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxnhymB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	nfpxyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));       
	npxnhyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxphyngmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	npxnhypgmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	QfmB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	rowdefaultB = (int*) malloc(sizeof(int) * (stepx*stepy));
	coldefaultB = (int*) malloc(sizeof(int) * (stepx*stepy));
	rowrentB = (int*) malloc(sizeof(int) * (stepx*stepy));
	colrentB = (int*) malloc(sizeof(int) * (stepx*stepy));
	rowtimedefaultB = (int*) malloc(sizeof(int) * (stepx+2));
	coltimedefaultB = (int*) malloc(sizeof(int) * (stepx+2));
		
/* 
* Grid generation parameters
*/
	/* State variables */
	myvar *w;
	myvar *p;		
	myvar *wealth;
	myvar *xom;
	myvar *xr;		
	myvar *xd;
	myvar *y;	
	int *ominx;		
	int *ominxdG;	
	int *ominxdB;	
	int *ominxdefG;	
	int *ominxdefB;
	int *rinxdefG;	
	int *rinxdefB;	
	int *ominy;	
	int *rinx;
	int *riny;
	int *nogriddefG;
	int **ominxdmaG;
	int *indicatiG;
	int *indicatjG;
	int **prerowdefaultmaG;
	int **precoldefaultmaG;
	int *prerowdefaultG;
	int *precoldefaultG;
	int **prerowrentmaG;
	int **precolrentmaG;
	int *prerowrentG;
	int *precolrentG;
	int *nogriddefB;
	int **ominxdmaB;
	int *indicatiB;
	int *indicatjB;
	int **prerowdefaultmaB;
	int **precoldefaultmaB;
	int *prerowdefaultB;
	int *precoldefaultB;
	int **prerowrentmaB;
	int **precolrentmaB;
	int *prerowrentB;
	int *precolrentB;

	w = (myvar*) malloc(sizeof(myvar) * (stepx));
	p = (myvar*) malloc(sizeof(myvar) * (stepy));		
	wealth = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	xom = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));
	xr = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));		
	xd = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	y = (myvar*) malloc(sizeof(myvar) * (stepx*stepy));	
	ominx = (int*) malloc(sizeof(int) * (stepx*stepy));		
	ominxdG = (int*) malloc(sizeof(int) * (stepx*stepy));	
	ominxdB = (int*) malloc(sizeof(int) * (stepx*stepy));	
	ominxdefG = (int*) malloc(sizeof(int) * (stepx*stepy));
	ominxdefB = (int*) malloc(sizeof(int) * (stepx*stepy));
	rinxdefG = (int*) malloc(sizeof(int) * (stepx*stepy));
	rinxdefB = (int*) malloc(sizeof(int) * (stepx*stepy));
	ominy = (int*) malloc(sizeof(int) * (stepx*stepy));	
	rinx = (int*) malloc(sizeof(int) * (stepx*stepy));
	riny = (int*) malloc(sizeof(int) * (stepx*stepy));

	
        nogriddefG = (int*) malloc(sizeof(int) * (stepx*stepy));
	for (i = 1; i <= stepx*stepy; i++) 
	{
		nogriddefG[i] = 1;
	}
	
    /*printf("FASE 0 \n");*/
	ominxdmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ominxdmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	indicatiG = (int*) malloc(sizeof(int*) * (stepx));
	indicatjG = (int*) malloc(sizeof(int*) * (stepy));
	
	prerowdefaultG = (int*) malloc(sizeof(int) * (stepx*stepy));
	precoldefaultG = (int*) malloc(sizeof(int) * (stepx*stepy));
	prerowrentG = (int*) malloc(sizeof(int) * (stepx*stepy));
	precolrentG = (int*) malloc(sizeof(int) * (stepx*stepy));

	prerowdefaultmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		prerowdefaultmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

        precoldefaultmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		precoldefaultmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	prerowrentmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		prerowrentmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

        precolrentmaG = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		precolrentmaG[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

	nogriddefB = (int*) malloc(sizeof(int) * (stepx*stepy));
	for (i = 1; i <= stepx*stepy; i++) 
	{
		nogriddefB[i] = 1;
	}
	
        ominxdmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		ominxdmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	indicatiB = (int*) malloc(sizeof(int*) * (stepx));
	indicatjB = (int*) malloc(sizeof(int*) * (stepy));
	
	prerowdefaultB = (int*) malloc(sizeof(int) * (stepx*stepy));
	precoldefaultB = (int*) malloc(sizeof(int) * (stepx*stepy));
	prerowrentB = (int*) malloc(sizeof(int) * (stepx*stepy));
	precolrentB = (int*) malloc(sizeof(int) * (stepx*stepy));

	prerowdefaultmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		prerowdefaultmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

        precoldefaultmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		precoldefaultmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}
	
	prerowrentmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		prerowrentmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

        precolrentmaB = (int**) malloc(sizeof(int*) * (stepx+2));
	for(i=0; i<= (stepx+2); i++)
	{
		precolrentmaB[i] = (int*)malloc(sizeof(int) * (stepx+2) );
	}

	
	myvar *vectime1;
	int *timeindicator;
	int *timeindicator2;
	myvar *vectime2;
	myvar *StorevaluesccxomG;
	myvar *StorevaluesccxomB;
        myvar *StorevaluesccxrG;
	myvar *StorevaluesccxrB;
	myvar *StorevaluesominxdG;
	myvar *StorevaluesominxdB;
        myvar *StorevaluesominxdefG;
	myvar *StorevaluesominxdefB;
	myvar *StorevaluesrinxdefG;
	myvar *StorevaluesrinxdefB;
	myvar *ProbdefiterG;
	myvar *ProbdefiterB;
	
	timeindicator = (int*) malloc(sizeof(int) * (stept));
	timeindicator2 = (int*) malloc(sizeof(int) * (stept2));
	vectime1 = (myvar*) malloc(sizeof(myvar) * (stept));
	vectime2 = (myvar*) malloc(sizeof(myvar) * (stept2));
	StorevaluesccxomG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesccxomB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
        StorevaluesccxrG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesccxrB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesominxdG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesominxdB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
        StorevaluesominxdefG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesominxdefB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesrinxdefG = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	StorevaluesrinxdefB = (myvar*) malloc(sizeof(myvar) * (stepx*stepy*stept2));
	ProbdefiterG = (myvar*) malloc(sizeof(myvar) * (stept2)); 
	ProbdefiterB = (myvar*) malloc(sizeof(myvar) * (stept2)); 
	
	for (i = 2; i <= stept; i++)
	{
		vectime1[i] = vectime1[i-1] + tau;
	}
	
	for (i = 2; i <= stept2; i++)
	{
		vectime2[i] = vectime2[i-1] + tau2;
	}
	
	timeindicator[1] = 1;
	timeindicator2[1] = 1;
	
	for (i = 2; i <= stept2; i++)
	{
		for (j = 2; j <= stept; j++)
		{
			if ( vectime1[j-1] < vectime2[i] & vectime2[i] <= vectime1[j] ) 
			{	
				timeindicator[j] = i;
				timeindicator2[i] = j;
				break;
			}
		}
	}	
		
/*
* Grid generation
*/	    
	printf("Create Grid\n");
	
	gridgen(stepx, stepy, hw, hp, wmin, wmax, pmin, pmax, w, p, H,
		    xom, xr, y, ominx, ominxdG, ominxdB, ominy, rinx, riny );
	
	for (i = 1; i <= stepx; i++)
	{
		if (w[i] <= posxom )
		{
			posgridxom = i;
		}
		else
		{
			break;
		}	
	}	
	
	for (i = 1; i <= stepx; i++)
	{
		if (w[i] <= posxr )
		{
			posgridxr = i;
		}
		else
		{
			break;
		}	
	}

	for (i = 1; i <= stepy; i++)
	{
		if (p[i] <= posy )
		{
			posgridy = i;
		}
		else
		{
			break;
		}	
	}

	printf("Min p = %3f\n",p[1]);
	printf("Max p = %3f\n",p[stepy]);
	printf("Min w = %3f\n",w[1]);
	printf("Max w = %3f\n",w[stepx]);
	printf("Cou = %3f\n",Cou);
	printf("Mo = %3f\n",Mo);
	printf("Pos x = %3u\n",posgridxom);
	printf("Pos y = %3u\n",posgridy);
    printf("Down payment = %3f\n",kappa);
    printf("lambda = %3f\n",lambda);
	printf("spread = %3f\n",spread);
	
	for (i = 1; i <= stepx*stepy; i++)
	{ 
		 if (ominx[i] == 1 && ominy[i] > 1 && ominy[i] <= stepy )
		 {
			 iaomG[i] = xom[i];
			 iaomB[i] = xom[i];
			 ccxomtimeG[i] = (xom[i]  + y[i]*H);
			 ccxomtimeB[i] = (xom[i]  + y[i]*H);
		 }
		else if (ominx[i] >= 1 && ominx[i] <= stepx && ominy[i] == 1 )
		{
			iaomG[i] = xom[i];
			iaomB[i] = xom[i];
			ccxomtimeG[i] = (xom[i] + y[i]*H);
			ccxomtimeB[i] = (xom[i]  + y[i]*H);
		}
		else
		{
			iaomG[i] = xom[i];
			iaomB[i] = xom[i];
			ccxomtimeG[i] = (xom[i] + y[i]*H);
			ccxomtimeB[i] = (xom[i] + y[i]*H);
		 }
		
		/* Owning - Good State */
		if (ominx[i] == 1 && ominy[i] > 1 && ominy[i] <= stepy )
		{
			VomtimeG[i] = ratgamma*( pow( xom[i] + H*y[i] ,(1-gamma)) );
		}
		else
		{	
			VomtimeG[i] = ratgamma*( pow( xom[i] + H*y[i] ,(1-gamma)) );
		}
		
		if (ominx[i] == 1 && ominy[i] > 1 && ominy[i] <= stepy )
		{ 
		        VomtimeB[i] = ratgamma*( pow( xom[i] + H*y[i] ,(1-gamma)) );
		}
		else
		{	
			VomtimeB[i] = ratgamma*( pow( xom[i] + H*y[i] ,(1-gamma)) );
		}
		
		iarG[i] = xr[i];
		iarB[i] = xr[i];
		ccxrtimeG[i] = xr[i];
		ccxrtimeB[i] = xr[i];
		
		/* Renting - Good State */
		VrtimeG[i] =ratgamma*( pow( xom[i]  ,(1-gamma)) );
		
		/* Renting - Bad State */
		VrtimeB[i] = ratgamma*( pow( xr[i] ,(1-gamma)) );
		
		/* Mortgage - Good State */
		div[i] = Cou;
		MtimeG[i] = 0;
		
		/* Mortgage - Bad State */
		MtimeB[i] = 0;
		
		/* Probability default - Good State */
		PDomtimeG[i] = 0;
		
		/* Probability default - Bad State */
		PDomtimeB[i] = 0;
	}

	/* Fixed Markov Probabilities - Renting - Good State */
	rnfpmarkov(    r, delta, taxhouse, lambda, 
				 mup, sigmap, 
			   stepx, stepy, invtau, invhw, invhp,
				  xr, y, H,
				QfrvG, nfpxphyrvG, nfpxnhyrvG, npxypgrvG, npxyngrvG, 
				npxphypgrvG, npxnhyngrvG, npxphyngrvG, npxnhypgrvG); 

	/* Fixed Markov Probabilities - Renting - Bad State */
	rnfpmarkov(   r,  delta, taxhouse, lambda, 
				mup, sigmap, 
			   stepx, stepy, invtau, invhw, invhp,
				 xr, y, H,
				QfrvB, nfpxphyrvB, nfpxnhyrvB, npxypgrvB, npxyngrvB, 
				npxphypgrvB, npxnhyngrvB, npxphyngrvB, npxnhypgrvB); 
	
	/* Fixed Markov Probabilities - Owning - Good State*/
	omnfpmarkov(     r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
			 xom, y, H, Cou, delta, taxhouse, taxrate, 
				 QfomvG, nfpxphyomvG, nfpxnhyomvG, npxypgomvG, npxyngomvG, 
				 npxphypgomvG, npxnhyngomvG, npxphyngomvG, npxnhypgomvG);
	
	/* Fixed Markov Probabilities - Owning - Bad State*/
	omnfpmarkov(     r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
				   xom, y, H, Cou,  delta, taxhouse, taxrate, 
				 QfomvB, nfpxphyomvB, nfpxnhyomvB, npxypgomvB, npxyngomvB, 
				 npxphypgomvB, npxnhyngomvB, npxphyngomvB, npxnhypgomvB);


	/* Fixed Markov Probabilities - Mortgage - Good State */
	mornfpmarkov(    r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
				  xom, y, H, Cou, delta, taxhouse, taxrate,
				 QfmG, nfpxphymG, nfpxnhymG, npxypgmG, npxyngmG, 
				 npxphypgmG, npxnhyngmG, npxphyngmG, npxnhypgmG);
	
	/* Fixed Markov Probabilities - Mortgage - Bad State */
	mornfpmarkov(    r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
				  xom, y, H, Cou,  delta, taxhouse, taxrate, 
				 QfmB, nfpxphymB, nfpxnhymB, npxypgmB, npxyngmB, 
				 npxphypgmB, npxnhyngmB, npxphyngmB, npxnhypgmB);
	
	/* Fixed Markov Probabilities - Probability default - Good State */
	pdnfpmarkov(    r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
				  xom, y, H, Cou,  delta, taxhouse, taxrate, 
				 PDQfomvG, PDnfpxphyomvG, PDnfpxnhyomvG, PDnpxypgomvG, PDnpxyngomvG, 
				 PDnpxphypgomvG, PDnpxnhyngomvG, PDnpxphyngomvG, PDnpxnhypgomvG);
	
	/* Fixed Markov Probabilities - Probability default - Bad State */
	pdnfpmarkov(    r, mup, sigmap, 
			     stepx, stepy, invtau, invhw, invhp,
				  xom, y, H, Cou,  delta, taxhouse, taxrate, 
				 PDQfomvB, PDnfpxphyomvB, PDnfpxnhyomvB, PDnpxypgomvB, PDnpxyngomvB, 
				 PDnpxphypgomvB, PDnpxnhyngomvB, PDnpxphyngomvB, PDnpxnhypgomvB);																																		
	
	//
// Value iteration - Optimal policy
//
	
	// Time iteration
	for (time = stept-1; time >= 1; time-- ) 
	{
	  /*printf("Time iteration = %3u of %3f\n",time,stept);
		/* Fixed Rate Mortgage */
	        myvar phi = (R*exp(- R*(T - time*tau)))/(1 - exp(- R*(T - time*tau)));
		myvar fac = (1 - exp(- R*(T - time*tau)))/(1 - exp(- R*T ));
		myvar Mt = fac*Mo;
		myvar facr = (1 - exp(- r*(T - time*tau)))/(1 - exp(- r*T ));
		myvar Mtr = facr*Mo;
		myvar Mofree = Cou*(1 - exp(- r*(T - time*tau) )) / r;
		myvar LGtime = LG[time];
		myvar LBtime = LB[time];
        	
		
		/****************************************************************************************************************************/		
		/**************************************     Renting   ***********************************************************************/
		/****************************************************************************************************************************/		
		
		/* Renting */
		/* printf("Renting - Value Function\n");*/
		type = 0;
		/* Renting */
		for (i = 1; i <= stepx*stepy; i++)
		{
			/* Good State */
			ccxroldG[rinx[i]+1][riny[i]+1] = ccxrtimeG[i];
			VroldG[rinx[i]+1][riny[i]+1] = VrtimeG[i];
			/* Bad State */
			ccxroldB[rinx[i]+1][riny[i]+1] = ccxrtimeB[i];
			VroldB[rinx[i]+1][riny[i]+1] = VrtimeB[i];
		}
		
		for (it = 0; it < imax; it++)
		{
			
			for ( i = stepx*stepy; i > 0; i--)
			{
				/* Good State */
				cconsrG[i] = ccxroldG[rinx[i]+1][riny[i]+1];
				dccxrxG[i] = invhw*(ccxroldG[rinx[i]+1][riny[i]+1] - ccxroldG[rinx[i]][riny[i]+1]);
				cfacrG[i] = dccxrxG[i]*dccxrxG[i];
				/* Bad State */
				cconsrB[i] = ccxroldB[rinx[i]+1][riny[i]+1];
				dccxrxB[i] = invhw*(ccxroldB[rinx[i]+1][riny[i]+1] - ccxroldB[rinx[i]][riny[i]+1]);
				cfacrB[i] = dccxrxB[i]*dccxrxB[i];
			}
			
			/* printf("Iteration = %3u of %3u\n",it,imax);
			 /* Value Iteration */
				consrent(  mup, sigmap, gamma, alpha, beta, H, r, rho,
					 tau, LGtime, invtau,
					 cconsrG, cfacrG,
					 ccxrG, ccxroldG, ccxrtimeG, ccxroldB, thetaG,
					 stepx, stepy, invhw, invhp, 
					 rinx, riny, xr, y, ucrcG, iarG, 
					 QfrcG, dtxircG, discircG,
					 nfpxnhyrcG, nfpxphyrcG, npxnhyrcG, 
					 npxphyrcG, npxypgrcG, npxyngrcG, 
					 npxphypgrcG, npxnhyngrcG, npxnhypgrcG, npxphyngrcG,
					 ptaurcG, pxypgrcG, pxphyrcG, pxnhyrcG, pxyngrcG,
					 pxphypgrcG, pxnhyngrcG, pxnhypgrcG, 
					 pxphyngrcG, pxyrcG, invprrcG);
			
			changeG = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				temp = ccxrG[rinx[i]+1][riny[i]+1] - ccxroldG[rinx[i]+1][riny[i]+1];
				changeG += temp*temp;
			}
			
			consrent(  mup, sigmap, gamma, alpha, beta, H, r, rho,
					 tau, LBtime, invtau,
					 cconsrB, cfacrB,
					 ccxrB, ccxroldB, ccxrtimeB, ccxroldG, thetaB,
					 stepx, stepy, invhw, invhp, 
					 rinx, riny, xr, y, ucrcB, iarB, 
					 QfrcB, dtxircB, discircB,
					 nfpxnhyrcB, nfpxphyrcB, npxnhyrcB, 
					 npxphyrcB, npxypgrcB, npxyngrcB, 
					 npxphypgrcB, npxnhyngrcB, npxnhypgrcB, npxphyngrcB,
					 ptaurcB, pxypgrcB, pxphyrcB, pxnhyrcB, pxyngrcB,
					 pxphypgrcB, pxnhyngrcB, pxnhypgrcB, 
					 pxphyngrcB, pxyrcB, invprrcB);
			
			changeB = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				temp = ccxrB[rinx[i]+1][riny[i]+1] - ccxroldB[rinx[i]+1][riny[i]+1];
				changeB += temp*temp;
			}
			
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				ccxroldG[rinx[i]+1][riny[i]+1] = ccxrG[rinx[i]+1][riny[i]+1];
				ccxroldB[rinx[i]+1][riny[i]+1] = ccxrB[rinx[i]+1][riny[i]+1];
			}
			
		}
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			/* Good State */
			ccxrtimeG[i] = ccxrG[rinx[i]+1][riny[i]+1];
			/* Bad State */
			ccxrtimeB[i] = ccxrB[rinx[i]+1][riny[i]+1];
		}
		
		// Step iteration
		for (it = 0; it < imax; it++)
		{
			
				/*printf("Iteration = %3u of %3u\n",it,imax);
			/* Value Iteration - Good State*/
			valuerent(   type, gamma, beta, LGtime, H, r, 
						invtau, stepx, stepy, invhw, rho, rinx, riny,
						VrG, VroldG, VrtimeG, VroldB, thetaG,
						ucrvG, iarG, ccxrtimeG,
						QfrvG, dtxirvG, discirvG,
						nfpxnhyrvG, nfpxphyrvG, npxnhyrvG, npxphyrvG, npxypgrvG, 
						npxyngrvG, npxphypgrvG, npxnhyngrvG, npxnhypgrvG, npxphyngrvG,
						ptaurvG, pxypgrvG, pxphyrvG, pxnhyrvG, pxyngrvG,
						pxphypgrvG, pxnhyngrvG, pxnhypgrvG, 
						pxphyngrvG, pxyrvG, invprrvG);
				
			changeG = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				temp = VrG[rinx[i]+1][riny[i]+1] - VroldG[rinx[i]+1][riny[i]+1];
				changeG += temp*temp;
			}
			
			/* Value Iteration - Bad State*/
			valuerent(   type, gamma, beta, LBtime, H, r, 
						invtau, stepx, stepy, invhw, rho, rinx, riny,
						VrB, VroldB, VrtimeB, VroldG, thetaB,
						ucrvB, iarB, ccxrtimeB,
						QfrvB, dtxirvB, discirvB,
						nfpxnhyrvB, nfpxphyrvB, npxnhyrvB, npxphyrvB, npxypgrvB, 
						npxyngrvB, npxphypgrvB, npxnhyngrvB, npxnhypgrvB, npxphyngrvB,
						ptaurvB, pxypgrvB, pxphyrvB, pxnhyrvB, pxyngrvB,
						pxphypgrvB, pxnhyngrvB, pxnhypgrvB, 
						pxphyngrvB, pxyrvB, invprrvB);
						
			changeB = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				temp = VrB[rinx[i]+1][riny[i]+1] - VroldB[rinx[i]+1][riny[i]+1];
				changeB += temp*temp;
			}
		
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
		
			for (i = 1; i <= stepx*stepy; i++)
			{
				VroldG[rinx[i]+1][riny[i]+1] = VrG[rinx[i]+1][riny[i]+1];
				VroldB[rinx[i]+1][riny[i]+1] = VrB[rinx[i]+1][riny[i]+1];
			}
			
		}
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			/* Good State */
			VrtimeG[i] = VrG[rinx[i]+1][riny[i]+1];
			/* Bad State */
			VrtimeB[i] = VrB[rinx[i]+1][riny[i]+1];
		}	
		
		/****************************************************************************************************************************/		
		/**************************************     Owning    ***********************************************************************/
		/****************************************************************************************************************************/		
		
		for (j = 1; j <= stepy; j++)
		{		
			for (i = stepx; i > 0; i--)
			{
				ind = i + stepx*(j - 1);
				
				/* Good State */
				if (rowdefaultG[ind] == 1 && rowrentG[ind] > 0)
				{
					VomoldG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = VrG[rowrentG[ind]+1][colrentG[ind]+1];
					ccxomoldG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = wmin;
					VomG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = VrG[rowrentG[ind]+1][colrentG[ind]+1];
					ccxomG[rowdefaultG[ind]+1][coldefaultG[ind]+1] =  wmin;
				}
				else if (rowdefaultG[ind] > 1 && rowrentG[ind] > 0)
				{
					VomoldG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = VrG[rowrentG[ind]+1][colrentG[ind]+1];
					ccxomoldG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = min_d(ccxrG[rowrentG[ind]+1][colrentG[ind]+1],xr[ind]);
					VomG[rowdefaultG[ind]+1][coldefaultG[ind]+1] = VrG[rowrentG[ind]+1][colrentG[ind]+1];
					ccxomG[rowdefaultG[ind]+1][coldefaultG[ind]+1] =  min_d(ccxrG[rowrentG[ind]+1][colrentG[ind]+1],xr[ind]);			
				}
				else if (rowdefaultG[ind] > 0 && rowrentG[ind] <= 0)
				{
					VomoldG[i+1][j+1] = VomG[i+2][j+1];
					ccxomoldG[i+1][j+1] = ccxomG[i+2][j+1];
					VomG[i+1][j+1] = VomG[i+2][j+1];
					ccxomG[i+1][j+1] = ccxomG[i+2][j+1];				
				}
				else
				{	
					ccxomoldG[ominx[ind]+1][ominy[ind]+1] = ccxomtimeG[ind];
					VomoldG[ominx[ind]+1][ominy[ind]+1] = VomtimeG[ind];
					VomtimeoldG[ind] = VomtimeG[ind];
				}
			}
		}
		
		for (j = 1; j <= stepy; j++)
		{		
			for (i = stepx; i > 0; i--)
			{
				ind = i + stepx*(j - 1);
				
				/* Good State */
				if (rowdefaultB[ind] == 1 && rowrentB[ind] > 0)
				{
					VomoldB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = VrB[rowrentB[ind]+1][colrentB[ind]+1];
					ccxomoldB[rowdefaultB[ind]+1][coldefaultB[ind]+1] =  wmin;
					VomB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = VrB[rowrentB[ind]+1][colrentB[ind]+1];
					ccxomB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = wmin;			
				}
				else if (rowdefaultB[ind] > 1 && rowrentB[ind] > 0)
				{
					VomoldB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = VrB[rowrentB[ind]+1][colrentB[ind]+1];
					ccxomoldB[rowdefaultB[ind]+1][coldefaultB[ind]+1] =  min_d(ccxrB[rowrentB[ind]+1][colrentB[ind]+1],xr[ind]);
					VomB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = VrB[rowrentB[ind]+1][colrentB[ind]+1];
					ccxomB[rowdefaultB[ind]+1][coldefaultB[ind]+1] = min_d(ccxrB[rowrentB[ind]+1][colrentB[ind]+1],xr[ind]);			
				}
				else if (rowdefaultB[ind] > 0 && rowrentB[ind] <= 0)
				{
					VomoldB[i+1][j+1] = VomB[i+2][j+1];
					ccxomoldB[i+1][j+1] = ccxomB[i+2][j+1];
					VomB[i+1][j+1] = VomB[i+2][j+1];
					ccxomB[i+1][j+1] = ccxomB[i+2][j+1];				
				}
				else
				{	
					ccxomoldB[ominx[ind]+1][ominy[ind]+1] = ccxomtimeB[ind];
					VomoldB[ominx[ind]+1][ominy[ind]+1] = VomtimeB[ind];
					VomtimeoldB[ind] = VomtimeB[ind];
				}
			}
		}
				
		for (it = 0; it < 30; it++)
		{
			for ( i = 1; i <= stepx*stepy;  i++)
			{
				/* Good State */
				cconsomG[i] = ccxomoldG[ominx[i]+1][ominy[i]+1];
				if ( ominy[i] == stepy )
				{
					dccxomyG[i] = 0;
					rcconsyG[i] = dccxomyG[i]/ccxomoldG[ominx[i]+1][ominy[i]+1],0;
				}
				else
				{
					dccxomyG[i] = invhp*(ccxomoldG[ominx[i]+1][ominy[i]+2] - ccxomoldG[ominx[i]+1][ominy[i]+1]);
					rcconsyG[i] = dccxomyG[i]/ccxomoldG[ominx[i]+1][ominy[i]+1];
				}
				
				
				/* Bad State */
				cconsomB[i] = ccxomoldB[ominx[i]+1][ominy[i]+1];
				if ( ominy[i] == stepy )
				{
					dccxomyB[i] = 0;
					rcconsyB[i] = dccxomyB[i]/ccxomoldB[ominx[i]+1][ominy[i]+1];
				}
				else
				{
					dccxomyB[i] = invhp*(ccxomoldB[ominx[i]+1][ominy[i]+2] - ccxomoldB[ominx[i]+1][ominy[i]+1]);
					rcconsyB[i] = dccxomyB[i]/ccxomoldB[ominx[i]+1][ominy[i]+1];
				}
			}
			
			/*printf("Iteration = %3u of %3u\n",it,imax); 
			 /* Value Iteration */
			
			consown(  mup, sigmap, gamma, alpha, beta, H, r, rho,
					tau, LGtime, invtau,
				        Mt, phi, Cou, taxrate, taxhouse, delta,
					cconsomG, rcconsxG, rcconsyG, 
					ccxomG, ccxomoldG, ccxomtimeG, ccxomoldB, thetaG,
					stepx, stepy, invhw, invhp, 
					ominx, ominxdG, ominy, xom, y, ucomcG, iaomG, 
					QfomcG, dtxiomcG, disciomcG,
					nfpxnhyomcG, nfpxphyomcG, npxnhyomcG, 
					npxphyomcG, npxypgomcG, npxyngomcG, 
					npxphypgomcG, npxnhyngomcG, npxnhypgomcG, npxphyngomcG,
					ptauomcG, pxypgomcG, pxphyomcG, pxnhyomcG, pxyngomcG,
					pxphypgomcG, pxnhyngomcG, pxnhypgomcG, 
					pxphyngomcG, pxyomcG, invpromcG);
			
			/* Interpolation filter */ 
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ccxomG[ominx[i]+1][ominy[i]+1]  < 0 )
				{	
					/*printf("Negative = (%3u,%3f,%3u,%3u) \n",i,ccxomG[ominx[i]+1][ominy[i]+1],ominx[i]+1,ominy[i]+1);*/
					
					for (j = ominx[i]+2; j <= stepy; j++)
					{
						if( ccxomG[j][ominy[i]+1]  > 0 )
						{	
							ccxomG[ominx[i]+1][ominy[i]+1] = ccxomG[ominx[i]][ominy[i]+1] + (xom[i] - xom[i-1])*
								((ccxomG[j][ominy[i]+1] - ccxomG[ominx[i]][ominy[i]+1])/(xom[i+ j - (ominx[i]+1) ] - xom[i-1]));
							ccxomoldG[ominx[i]+1][ominy[i]+1] = ccxomG[ominx[i]+1][ominy[i]+1];
							break;
						}	
					}
					/*printf("Correction = (%3f,%3f,%3f,%3u) \n",ccxomG[ominx[i]+1][ominy[i]+1],ccxomG[j][ominy[i]+1],xom[i+ j - (ominx[i]+1) ],j);*/
				}	
			}
			
			
			changeG = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					temp = ccxomG[ominx[i]+1][ominy[i]+1] - ccxomoldG[ominx[i]+1][ominy[i]+1];
					changeG += temp*temp;
				}	
			}
			
			consown(  mup, sigmap, gamma, alpha, beta, H, r, rho,
					tau, LBtime, invtau,
				        Mt, phi, Cou, taxrate, taxhouse, delta,
					cconsomB, rcconsxB, rcconsyB, 
					ccxomB, ccxomoldB, ccxomtimeB, ccxomoldG, thetaB,
					stepx, stepy, invhw, invhp, 
					ominx, ominxdB, ominy, xom, y, ucomcB, iaomB, 
					QfomcB, dtxiomcB, disciomcB,
					nfpxnhyomcB, nfpxphyomcB, npxnhyomcB, 
					npxphyomcB, npxypgomcB, npxyngomcB, 
					npxphypgomcB, npxnhyngomcB, npxnhypgomcB, npxphyngomcB,
					ptauomcB, pxypgomcB, pxphyomcB, pxnhyomcB, pxyngomcB,
					pxphypgomcB, pxnhyngomcB, pxnhypgomcB, 
					pxphyngomcB, pxyomcB, invpromcB);
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ccxomB[ominx[i]+1][ominy[i]+1]  < 0 )
				{	
					ccxomB[ominx[i]+1][ominy[i]+1] = ccxomB[ominx[i]][ominy[i]+1] + (xom[i] - xom[i-1])*
							((ccxomG[ominx[i]+2][ominy[i]+1] - ccxomB[ominx[i]][ominy[i]+1])/(xom[i+1] - xom[i-1]));
					ccxomoldB[ominx[i]+1][ominy[i]+1] = ccxomB[ominx[i]+1][ominy[i]+1];		
				}	
			}
			
			changeB = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{
					temp = ccxomB[ominx[i]+1][ominy[i]+1] - ccxomoldB[ominx[i]+1][ominy[i]+1];
					changeB += temp*temp;
				}	
			}
			
			/*printf("Change = (%3f,%3f) \n",changeG,changeB);*/
			
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					ccxomoldG[ominx[i]+1][ominy[i]+1] = ccxomG[ominx[i]+1][ominy[i]+1];
                                       
				}
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{
					ccxomoldB[ominx[i]+1][ominy[i]+1] = ccxomB[ominx[i]+1][ominy[i]+1];
                                       
				}
			}
		}
		
		type = 0;
		
		// Step iteration
		for (it = 0; it < imax; it++)
		{
				/* printf("Iteration = %3u of %3u\n",it,imax);
			/* Value Iteration - Good State*/
		       valueown(  type, gamma, beta, LGtime, H, r, Cou, taxrate, wmax, invtau, 
				   stepx, stepy, invhw, phi, rho, Mt, y, xom,
						ominx, ominxdG, ominy, 
						VomG, VomoldG, VomtimeG, VomoldB, thetaG,
				                ucomvG, iaomG, ccxomG, QfomvG, dtxiomvG, disciomvG,
						nfpxnhyomvG, nfpxphyomvG, npxnhyomvG, npxphyomvG,
						npxypgomvG, npxyngomvG, npxphypgomvG, npxnhyngomvG, npxnhypgomvG, npxphyngomvG,
						felicityvG, ptauomvG, pxypgomvG, pxphyomvG, pxnhyomvG, pxyngomvG,
						pxphypgomvG, pxnhyngomvG, pxnhypgomvG, pxphyngomvG, pxyomvG, invpromvG);
				
			changeG = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					temp = VomG[ominx[i]+1][ominy[i]+1] - VomoldG[ominx[i]+1][ominy[i]+1];
					changeG += temp*temp;
				}
			}
			
			/* Value Iteration - Bad State*/
		        valueown(  type, gamma, beta, LBtime, H, r, Cou, taxrate, wmax, invtau, 
				   stepx, stepy, invhw, phi, rho, Mt, y, xom,
						ominx, ominxdB, ominy, 
						VomB, VomoldB, VomtimeB, VomoldG, thetaB,
				                ucomvB, iaomB, ccxomB, QfomvB, dtxiomvB, disciomvB,
						nfpxnhyomvB, nfpxphyomvB, npxnhyomvB, npxphyomvB,
						npxypgomvB, npxyngomvB, npxphypgomvB, npxnhyngomvB, npxnhypgomvB, npxphyngomvB,
						felicityvB, ptauomvB, pxypgomvB, pxphyomvB, pxnhyomvB, pxyngomvB,
						pxphypgomvB, pxnhyngomvB, pxnhypgomvB, pxphyngomvB, pxyomvB, invpromvB);

						
			changeB = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{	
					temp = VomB[ominx[i]+1][ominy[i]+1] - VomoldB[ominx[i]+1][ominy[i]+1];
					changeB += temp*temp;
				}
			}
			
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					VomoldG[ominx[i]+1][ominy[i]+1] = VomG[ominx[i]+1][ominy[i]+1];
				}
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{	
					VomoldB[ominx[i]+1][ominy[i]+1] = VomB[ominx[i]+1][ominy[i]+1];
				}
			}
		}
		
		/****************************************************************************************************************************/		
		/**************************************     Default   ***********************************************************************/
		/****************************************************************************************************************************/		
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
		        xd[i] = xom[i] - epsilon*H*y[i];
			
			if( xd[i] > wmax + H*pmax)
			{
				xd[i] = wmax + H*pmax;
			}																
		}									
		
		/****************************************************************************************************************************/		
		/**************************************     Collateral   ***********************************************************************/
		/****************************************************************************************************************************/		
		
		for (i = 1; i <= stepx*stepy; i++)
		{
			collateral[i] = dead*H*y[i];
			div[i] = Cou;
		}	

		
		/* Good State */
		defaultind(     rowdefaultG, coldefaultG,
		rowdefaultmaG, coldefaultmaG,
		rowrentG, colrentG,
		rowrentmaG, colrentmaG,
		rowtimedefaultG, coltimedefaultG,
		ominx, ominy, ominxdG, ominxdefG, rinxdefG, rinx, riny,
		VomoldG, VomG, VrG, ccxomG, ccxrG, MtimeG, collateral, 
		wealth, xd, xom, xr, y, Mt, H,
		stepx, stepy, tresh, wmax, pmax,
		nogriddefG, ominxdmaG, indicatiG, indicatjG,
		prerowdefaultmaG, precoldefaultmaG, prerowdefaultG, precoldefaultG,
		prerowrentmaG, precolrentmaG, prerowrentG, precolrentG);
		 
		 /* Bad State */
		defaultind(     rowdefaultB, coldefaultB,
		rowdefaultmaB, coldefaultmaB,
		rowrentB, colrentB,
		rowrentmaB, colrentmaB,
		rowtimedefaultB, coltimedefaultB,
		ominx, ominy, ominxdB, ominxdefB, rinxdefB, rinx, riny,
		VomoldB, VomB, VrB, ccxomB, ccxrB, MtimeB, collateral,
		wealth, xd, xom, xr, y, Mt, H,
		stepx, stepy, tresh, wmax, pmax,
		nogriddefB, ominxdmaB, indicatiB, indicatjB,
		prerowdefaultmaB, precoldefaultmaB, prerowdefaultB, precoldefaultB,
		prerowrentmaB, precolrentmaB, prerowrentB, precolrentB);
		
				
		for (i = 1; i <= stepx*stepy; i++)				
		{
			VomtimeG[i] = VomG[ominx[i]+1][ominy[i]+1];
			VomtimeB[i] = VomB[ominx[i]+1][ominy[i]+1];
			
			ccxomtimeG[i] = ccxomG[ominx[i]+1][ominy[i]+1];
			ccxomtimeB[i] = ccxomB[ominx[i]+1][ominy[i]+1];
		}
		
		if ( timeindicator[time] > 0 )
		{
			
			for (i = 1; i <= stepx*stepy; i++)				
			{
				int posxstore = i + stepx*stepy*(timeindicator[time]-1);
				
				StorevaluesccxomG[posxstore] = ccxomtimeG[i];  
				StorevaluesccxomB[posxstore] = ccxomtimeB[i];
				StorevaluesccxrG[posxstore] = ccxrtimeG[i];  
				StorevaluesccxrB[posxstore] = ccxrtimeB[i];
				StorevaluesominxdG[posxstore] = ominxdG[i];  
				StorevaluesominxdB[posxstore] = ominxdB[i];
                        	StorevaluesominxdefG[posxstore] = ominxdefG[i];  
				StorevaluesominxdefB[posxstore] = ominxdefB[i];
				StorevaluesrinxdefG[posxstore] = rinxdefG[i];  
				StorevaluesrinxdefB[posxstore] = rinxdefB[i];
 
			}
		}
		
		
		/****************************************************************************************************************************/		
		/**************************************     Morgtage   **********************************************************************/
		/****************************************************************************************************************************/		
		
		type = 0;
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			if ( rowdefaultG[i] > 0 )
			{
				MoldG[ominx[i]+1][ominy[i]+1] =   MtimeG[i];
			}
			else
			{
				MoldG[ominx[i]+1][ominy[i]+1] = MtimeG[i];
			}
		}
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			if ( rowdefaultB[i] > 0 )
			{
				MoldB[ominx[i]+1][ominy[i]+1] =  MB[ominx[i]+1][ominy[i]+1];
			}
			else
			{
				MoldB[ominx[i]+1][ominy[i]+1] = MtimeB[i];
			}
		}		 
		
		
		for (it = 0; it < imax; it++)
		{
		
			machinemor(   type, gamma, beta, LGtime, H, r, Cou, taxrate, wmax, invtau, 
			MG, MoldG, MtimeG, MoldB, thetaG,
			stepx, stepy, invhw, phi, rho, Mt, Mofree,
			ominx, ominxdG, ominy, ucmG, iaomG, ccxomtimeG, 
			QfmG, dtximG, discimG,
			nfpxnhymG, nfpxphymG, npxnhymG, npxphymG,
			npxypgmG, npxyngmG, npxphypgmG, npxnhyngmG, npxnhypgmG, npxphyngmG,
			ptaumG, pxypgmG, pxphymG, pxnhymG, pxyngmG,
			pxphypgmG, pxnhyngmG, pxnhypgmG, pxphyngmG, pxymG, invprmG);
		
	         	machinemor(   type, gamma, beta, LBtime, H, r, Cou, taxrate, wmax, invtau, 
			MB, MoldB, MtimeB, MoldG, thetaB, 
			stepx, stepy, invhw, phi, rho, Mt, Mofree, 
			ominx, ominxdB, ominy, ucmB, iaomB, ccxomtimeB, 
			QfmB, dtximB, discimB,
			nfpxnhymB, nfpxphymB, npxnhymB, npxphymB,
			npxypgmB, npxyngmB, npxphypgmB, npxnhyngmB, npxnhypgmB, npxphyngmB,
			ptaumB, pxypgmB, pxphymB, pxnhymB, pxyngmB,
			pxphypgmB, pxnhyngmB, pxnhypgmB, pxphyngmB, pxymB, invprmB);

			changeG = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					temp = MG[ominx[i]+1][ominy[i]+1] - MoldG[ominx[i]+1][ominy[i]+1];
					changeG += temp*temp;
				}	
			}
			
			changeB = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{	
					temp = MB[ominx[i]+1][ominy[i]+1] - MoldB[ominx[i]+1][ominy[i]+1];
					changeB += temp*temp;
				}
			}
			
			/*printf("Mortgage change = (%3u,%3f,%3f)\n",it,changeG,changeB);*/
		
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
		
			
			
			for (i = 1; i <= stepx*stepy; i++)
			{
					
					MoldG[ominx[i]+1][ominy[i]+1] = MG[ominx[i]+1][ominy[i]+1];
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
					MoldB[ominx[i]+1][ominy[i]+1] = MB[ominx[i]+1][ominy[i]+1];
			}
			
		 }
		 
		 for (i = 1; i <= stepx*stepy; i++)				
		 {
			 MtimeG[i] = MG[ominx[i]+1][ominy[i]+1];
			 MtimeB[i] = MB[ominx[i]+1][ominy[i]+1];
		 }
		
		/****************************************************************************************************************************/		
		/**************************************     Probability default *************************************************************/
		/****************************************************************************************************************************/		
		
		type = 0;
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			if ( rowdefaultG[i] > 0 )
			{
				PDomoldG[ominx[i]+1][ominy[i]+1] = 1;
			}
			else
			{
				PDomoldG[ominx[i]+1][ominy[i]+1] = PDomtimeG[i];
			}
		}
		
		for (i = 1; i <= stepx*stepy; i++)				
		{
			if ( rowdefaultB[i] > 0 )
			{
				PDomoldB[ominx[i]+1][ominy[i]+1] = 1;
			}
			else
			{
				PDomoldB[ominx[i]+1][ominy[i]+1] = PDomtimeB[i];
			}
		}		 
		
		
		for (it = 0; it < imax; it++)
		{
		
			/*printf("Iteration  = %3u of %3u\n",it,imax);
			
		/*printf("G state \n");*/   
			machinePD(   type, gamma, beta, LGtime, H, r, Cou, taxrate, wmax, invtau, 
			PDomG, PDomoldG, PDomtimeG, PDomoldB, thetaG,
			stepx, stepy, invhw, phi, rho, Mt, Mofree,
			ominx, ominxdG, ominy, PDucomvG, PDiaomG, ccxomtimeG, 
			PDQfomvG, PDdtxiomvG, PDdisciomvG,
			PDnfpxnhyomvG, PDnfpxphyomvG, PDnpxnhyomvG, PDnpxphyomvG,
			PDnpxypgomvG, PDnpxyngomvG, PDnpxphypgomvG, PDnpxnhyngomvG, PDnpxnhypgomvG, PDnpxphyngomvG,
			PDptauomvG, PDpxypgomvG, PDpxphyomvG, PDpxnhyomvG, PDpxyngomvG,
			PDpxphypgomvG, PDpxnhyngomvG, PDpxnhypgomvG, PDpxphyngomvG, PDpxyomvG, PDinvpromvG);
		
		/*printf("B state \n");*/   
			machinePD(   type, gamma, beta, LBtime, H, r, Cou, taxrate, wmax, invtau, 
			PDomB, PDomoldB, PDomtimeB, PDomoldG, thetaB, 
			stepx, stepy, invhw, phi, rho, Mt, Mofree, 
			ominx, ominxdB, ominy, PDucomvB, PDiaomB, ccxomtimeB, 
			PDQfomvB, PDdtxiomvB, PDdisciomvB,
			PDnfpxnhyomvB, PDnfpxphyomvB, PDnpxnhyomvB, PDnpxphyomvB,
			PDnpxypgomvB, PDnpxyngomvB, PDnpxphypgomvB, PDnpxnhyngomvB, PDnpxnhypgomvB, PDnpxphyngomvB,
			PDptauomvB, PDpxypgomvB, PDpxphyomvB, PDpxnhyomvB, PDpxyngomvB,
			PDpxphypgomvB, PDpxnhyngomvB, PDpxnhypgomvB, PDpxphyngomvB, PDpxyomvB, PDinvpromvB);

			changeG = changestart;
			
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdG[i] > 0 )
				{	
					temp = PDomG[ominx[i]+1][ominy[i]+1] - PDomoldG[ominx[i]+1][ominy[i]+1];
					changeG += temp*temp;
				}	
			}
			
			changeB = changestart;
			for (i = 1; i <= stepx*stepy; i++)
			{
				if( ominxdB[i] > 0 )
				{	
					temp = PDomB[ominx[i]+1][ominy[i]+1] - PDomoldB[ominx[i]+1][ominy[i]+1];
					changeB += temp*temp;
				}
			}
			
			/*printf("Mortgage change = (%3u,%3f,%3f)\n",it,changeG,changeB);*/
		
			if (changeG < tol && changeB < tol) 
			{
				break;
			}
		
			
			
			for (i = 1; i <= stepx*stepy; i++)
			{
					
					PDomoldG[ominx[i]+1][ominy[i]+1] = PDomG[ominx[i]+1][ominy[i]+1];
			}
			
			for (i = 1; i <= stepx*stepy; i++)
			{
					PDomoldB[ominx[i]+1][ominy[i]+1] = PDomB[ominx[i]+1][ominy[i]+1];
			}
			
		 }
		 
		 for (i = 1; i <= stepx*stepy; i++)				
		 {
			 PDomtimeG[i] = PDomG[ominx[i]+1][ominy[i]+1];
			 PDomtimeB[i] = PDomB[ominx[i]+1][ominy[i]+1];
		 }
		 
		 ProbdefG[time] = PDomG[posgridxom+1][posgridy+1]; 
		 
		
		 /****************************************************************************************************************************/		
		/************************************** Default boundary  ********************************************************************/
		/*****************************************************************************************************************************/		
		
		
		for( j = 1; j <= stepy; j++)
		{
			for (i = 2; i <= stepx; i++)
			{	
				ind = i + (j-1)*stepx ;	
				inddef = 1 + (j-1)*6 + 6*stepy*(time - 1);
				
				if( rowdefaultG[ind] > 0 && coldefaultG[ind] == coldefaultG[ind-1] )
				{
					DefaultboundaryG[inddef] = rowdefaultG[ind]; 
					DefaultboundaryG[inddef+1] = coldefaultG[ind];
					DefaultboundaryG[inddef+2] = H*y[ind];
					DefaultboundaryG[inddef+3] = wealth[ind];
					DefaultboundaryG[inddef+4] = Mt;
					DefaultboundaryG[inddef+5] = PDomtimeG[ind];
				}	
			}
		}
		
		for( j = 1; j <= stepy; j++)
		{
			for (i = 2; i <= stepx; i++)
			{	
				ind = i + (j-1)*stepx ;	
				inddef = 1 + (j-1)*6 + 6*stepy*(time - 1);
				
				if( rowdefaultB[ind] > 0 && coldefaultB[ind] == coldefaultB[ind-1] )
				{
					DefaultboundaryB[inddef] = rowdefaultB[ind]; 
					DefaultboundaryB[inddef+1] = coldefaultB[ind];
					DefaultboundaryB[inddef+2] = H*y[ind];
					DefaultboundaryB[inddef+3] = wealth[ind];
					DefaultboundaryB[inddef+4] = Mt;
					DefaultboundaryB[inddef+5] = PDomtimeB[ind];
				}	
			}
		}

		
	}
	
	/****************************************************************************************************************************/		
	/************************************** Save outuput  ********************************************************************/
	/*****************************************************************************************************************************/		
	/* for (time = stept2-1; time >= 1; time-- )  */

	printf("\nCreating Output Files 1 \n");	
	
	fp = fopen(PARAM_FILE, "w+");
	for(i=1; i<=25; i++)
	{
		fprintf(fp,"%3.14f ",parameters[i]);
	}
	fclose(fp);	

	fp = fopen(StorevaluesccxomG_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
		
			fprintf(fp,"%3.14f ",StorevaluesccxomG[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesccxomB_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesccxomB[posxstore]);
		}
	}	
	fclose(fp);

	fp = fopen(StorevaluesccxrG_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ )  
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
		
			fprintf(fp,"%3.14f ",StorevaluesccxrG[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesccxrB_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesccxrB[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesominxdG_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesominxdG[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesominxdB_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ )  
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesominxdB[posxstore]);
		}
	}	
	fclose(fp);

        fp = fopen(StorevaluesominxdefG_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ )  
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesominxdefG[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesominxdefB_FILE, "w+");
        for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesominxdefB[posxstore]);
		}
	}	
	fclose(fp);

	fp = fopen(StorevaluesrinxdefG_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesrinxdefG[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(StorevaluesrinxdefB_FILE, "w+");
	for (time = 1; time <=  stept2-1; time++ ) 
	{
		for(i=1;i<=stepx*stepy;i++)
		{
			int posxstore = i + stepx*stepy*(time-1);
			
			fprintf(fp,"%3.14f ",StorevaluesrinxdefB[posxstore]);
		}
	}	
	fclose(fp);
	
	fp = fopen(PDOMG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",PDomG[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	fp = fopen(VOMG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",VomG[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	
	fp = fopen(PDOMB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",PDomB[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	fp = fopen(VOMB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",VomB[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	fp = fopen(VRG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",VrG[rinx[i]+1][riny[i]+1]);
	}
	fclose(fp);
	
	
	fp = fopen(VMORG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",MG[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	fp = fopen(VMORB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",MB[ominx[i]+1][ominy[i]+1]);
	}
	fclose(fp);
	
	fp = fopen(ROWMAG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",rowdefaultG[i]);
	}
	fclose(fp);
	
	fp = fopen(COLMAG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",coldefaultG[i]);
	}
	fclose(fp);	
	
	fp = fopen(ROWMAB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",rowdefaultB[i]);
	}
	fclose(fp);
	
	fp = fopen(COLMAB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",coldefaultB[i]);
	}
	fclose(fp);
	
	fp = fopen(OMINXDEFG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",ominxdefG[i]);
	}
	fclose(fp);
	
	fp = fopen(OMINXDEFB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",ominxdefB[i]);
	}
	fclose(fp);
	
	fp = fopen(RINXDEFG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",rinxdefG[i]);
	}
	fclose(fp);
	
	fp = fopen(RINXDEFB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",rinxdefB[i]);
	}
	fclose(fp);
	
	fp = fopen(OMINXDG_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",ominxdG[i]);
	}
	fclose(fp);
	
	fp = fopen(OMINXDB_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%6u ",ominxdB[i]);
	}
	fclose(fp);
	
	fp = fopen(XD_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",xd[i]);
	}
	fclose(fp);
	
	fp = fopen(XOM_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",xom[i]);
	}
	fclose(fp);
	
	fp = fopen(XR_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",xr[i]);
	}
	fclose(fp);
	
	fp = fopen(Y_FILE, "w+");
	for(i=1;i<=stepx*stepy;i++)
	{
		fprintf(fp,"%3.14f ",y[i]);
	}
	fclose(fp);
	
}
